#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.polyflow.matrix import V1Hyperband
from polyaxon.polytune.search_managers.hyperband.manager import HyperbandManager
from tests.utils import BaseTestCase


@pytest.mark.polytune_mark
class TestHyperbandSearchManager(BaseTestCase):
    DISABLE_RUNNER = True
    DISABLE_EXECUTOR = True
    DISABLE_AUDITOR = True

    def setUp(self):
        super().setUp()
        config = V1Hyperband.from_dict(
            {
                "concurrency": 2,
                "maxIterations": 10,
                "eta": 3,
                "resource": {"name": "steps", "type": "float"},
                "resume": False,
                "metric": {"name": "loss", "optimization": "minimize"},
                "params": {
                    "feature1": {"kind": "choice", "value": [1, 2, 3]},
                    "feature2": {"kind": "linspace", "value": [1, 2, 5]},
                    "feature3": {"kind": "range", "value": [1, 5, 1]},
                },
            }
        )
        self.manager1 = HyperbandManager(config=config)

        config = V1Hyperband.from_dict(
            {
                "concurrency": 2,
                "maxIterations": 81,
                "eta": 3,
                "resource": {"name": "size", "type": "int"},
                "resume": False,
                "metric": {"name": "loss", "optimization": "minimize"},
                "params": {
                    "feature1": {"kind": "choice", "value": [1, 2, 3]},
                    "feature2": {"kind": "linspace", "value": [1, 2, 5]},
                    "feature3": {"kind": "range", "value": [1, 5, 1]},
                    "feature4": {"kind": "range", "value": [1, 5, 1]},
                },
            }
        )
        self.manager2 = HyperbandManager(config=config)

    def test_hyperband_config(self):
        assert HyperbandManager.CONFIG == V1Hyperband

    @staticmethod
    def almost_equal(value, value_compare):
        assert value_compare - 0.02 <= value <= value_compare + 0.02

    def test_hyperband_properties(self):
        # Manager1
        assert self.manager1.config.max_iterations == 10
        assert self.manager1.config.eta == 3
        assert self.manager1.config.s_max == 2
        assert (
            self.manager1.config.B
            == (self.manager1.config.s_max + 1) * self.manager1.config.max_iterations
        )

        assert self.manager2.config.max_iterations == 81
        assert self.manager2.config.eta == 3
        assert self.manager2.config.s_max == 4
        assert (
            self.manager2.config.B
            == (self.manager2.config.s_max + 1) * self.manager2.config.max_iterations
        )

    def test_get_bracket(self):
        # Manager1
        assert self.manager1.config.get_bracket(iteration=0) == 2
        assert self.manager1.config.get_bracket(iteration=1) == 1
        assert self.manager1.config.get_bracket(iteration=2) == 0

        # Manager2
        assert self.manager2.config.get_bracket(iteration=0) == 4
        assert self.manager2.config.get_bracket(iteration=1) == 3
        assert self.manager2.config.get_bracket(iteration=2) == 2
        assert self.manager2.config.get_bracket(iteration=3) == 1
        assert self.manager2.config.get_bracket(iteration=4) == 0

    def test_get_num_runs(self):
        # Manager1
        assert self.manager1.config.get_num_runs(bracket=2) == 9
        assert self.manager1.config.get_num_runs(bracket=1) == 5
        assert self.manager1.config.get_num_runs(bracket=0) == 3

        # Manager2
        assert self.manager2.config.get_num_runs(bracket=4) == 81
        assert self.manager2.config.get_num_runs(bracket=3) == 34
        assert self.manager2.config.get_num_runs(bracket=2) == 15
        assert self.manager2.config.get_num_runs(bracket=1) == 8
        assert self.manager2.config.get_num_runs(bracket=0) == 5

    def test_get_num_runs_to_keep(self):
        # Number of config to keep [/n_suggestions, /iteration]

        # Manager1
        # Iteration == 0
        assert (
            self.manager1.config.get_num_runs_to_keep(num_runs=9, bracket_iteration=0)
            == 3
        )
        assert (
            self.manager1.config.get_num_runs_to_keep(num_runs=9, bracket_iteration=1)
            == 1
        )
        assert (
            self.manager1.config.get_num_runs_to_keep(num_runs=9, bracket_iteration=2)
            == 0
        )

        assert (
            self.manager1.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=0
            )
            == 3
        )
        assert (
            self.manager1.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=1
            )
            == 1
        )
        assert (
            self.manager1.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=2
            )
            == 0
        )

        # Iteration == 1
        assert (
            self.manager1.config.get_num_runs_to_keep(num_runs=5, bracket_iteration=0)
            == 1
        )
        assert (
            self.manager1.config.get_num_runs_to_keep(num_runs=5, bracket_iteration=1)
            == 0
        )

        assert (
            self.manager1.config.get_num_runs_to_keep_for_iteration(
                iteration=1, bracket_iteration=0
            )
            == 1
        )
        assert (
            self.manager1.config.get_num_runs_to_keep_for_iteration(
                iteration=1, bracket_iteration=1
            )
            == 0
        )

        # Iteration == 2
        assert (
            self.manager1.config.get_num_runs_to_keep(num_runs=3, bracket_iteration=0)
            == 1
        )
        assert (
            self.manager1.config.get_num_runs_to_keep_for_iteration(
                iteration=2, bracket_iteration=0
            )
            == 1
        )

        # Manager2
        # Iteration == 0
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=81, bracket_iteration=0)
            == 27
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=81, bracket_iteration=1)
            == 9
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=81, bracket_iteration=2)
            == 3
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=81, bracket_iteration=3)
            == 1
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=81, bracket_iteration=4)
            == 0
        )

        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=0
            )
            == 27
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=1
            )
            == 9
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=2
            )
            == 3
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=3
            )
            == 1
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=0, bracket_iteration=4
            )
            == 0
        )

        # Iteration == 1
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=34, bracket_iteration=0)
            == 11
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=34, bracket_iteration=1)
            == 3
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=34, bracket_iteration=2)
            == 1
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=34, bracket_iteration=3)
            == 0
        )

        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=1, bracket_iteration=0
            )
            == 11
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=1, bracket_iteration=1
            )
            == 3
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=1, bracket_iteration=2
            )
            == 1
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=1, bracket_iteration=3
            )
            == 0
        )

        # Iteration == 2
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=15, bracket_iteration=0)
            == 5
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=15, bracket_iteration=1)
            == 1
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=15, bracket_iteration=2)
            == 0
        )

        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=2, bracket_iteration=0
            )
            == 5
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=2, bracket_iteration=1
            )
            == 1
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=2, bracket_iteration=2
            )
            == 0
        )

        # Iteration == 3
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=8, bracket_iteration=0)
            == 2
        )
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=8, bracket_iteration=1)
            == 0
        )

        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=3, bracket_iteration=0
            )
            == 2
        )
        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=3, bracket_iteration=1
            )
            == 0
        )

        # Iteration == 4
        assert (
            self.manager2.config.get_num_runs_to_keep(num_runs=5, bracket_iteration=0)
            == 1
        )

        assert (
            self.manager2.config.get_num_runs_to_keep_for_iteration(
                iteration=4, bracket_iteration=0
            )
            == 1
        )

    def test_get_resources(self):
        # Number of resources [/bracket, /iteration]

        # Manager1
        self.almost_equal(self.manager1.get_resources(bracket=2), 1.11)
        self.almost_equal(self.manager1.get_resources(bracket=1), 3.33)
        self.almost_equal(self.manager1.get_resources(bracket=0), 10)

        self.almost_equal(self.manager1.get_resources_for_iteration(iteration=0), 1.11)
        self.almost_equal(self.manager1.get_resources_for_iteration(iteration=1), 3.33)
        self.almost_equal(self.manager1.get_resources_for_iteration(iteration=2), 10)

        # Manager2
        self.almost_equal(self.manager2.get_resources(bracket=4), 1)
        self.almost_equal(self.manager2.get_resources(bracket=3), 3)
        self.almost_equal(self.manager2.get_resources(bracket=2), 9)
        self.almost_equal(self.manager2.get_resources(bracket=1), 27)
        self.almost_equal(self.manager2.get_resources(bracket=0), 81)

        self.almost_equal(self.manager2.get_resources_for_iteration(iteration=0), 1)
        self.almost_equal(self.manager2.get_resources_for_iteration(iteration=1), 3)
        self.almost_equal(self.manager2.get_resources_for_iteration(iteration=2), 9)
        self.almost_equal(self.manager2.get_resources_for_iteration(iteration=3), 27)
        self.almost_equal(self.manager2.get_resources_for_iteration(iteration=4), 81)

    def test_get_n_resources(self):
        # Number of iteration resources

        # Manager1
        # Iteration == 0
        self.almost_equal(
            self.manager1.get_n_resources(n_resources=1.11, bracket_iteration=0), 1.11
        )
        self.almost_equal(
            self.manager1.get_n_resources(n_resources=1.11, bracket_iteration=1), 3.33
        )
        self.almost_equal(
            self.manager1.get_n_resources(n_resources=1.11, bracket_iteration=2), 9.99
        )

        self.almost_equal(
            self.manager1.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=0
            ),
            1.11,
        )
        self.almost_equal(
            self.manager1.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=1
            ),
            3.33,
        )
        self.almost_equal(
            self.manager1.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=2
            ),
            9.99,
        )

        # Iteration == 1
        self.almost_equal(
            self.manager1.get_n_resources(n_resources=3.33, bracket_iteration=0), 3.33
        )
        self.almost_equal(
            self.manager1.get_n_resources(n_resources=3.33, bracket_iteration=1), 9.99
        )

        self.almost_equal(
            self.manager1.get_n_resources_for_iteration(
                iteration=1, bracket_iteration=0
            ),
            3.33,
        )
        self.almost_equal(
            self.manager1.get_n_resources_for_iteration(
                iteration=1, bracket_iteration=1
            ),
            9.99,
        )

        # Iteration == 2
        self.almost_equal(
            self.manager1.get_n_resources(n_resources=9.99, bracket_iteration=0), 9.99
        )

        self.almost_equal(
            self.manager1.get_n_resources_for_iteration(
                iteration=2, bracket_iteration=0
            ),
            9.99,
        )

        # Manager2
        # Iteration == 0
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=1, bracket_iteration=0), 1
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=1, bracket_iteration=1), 3
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=1, bracket_iteration=2), 9
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=1, bracket_iteration=3), 27
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=1, bracket_iteration=4), 81
        )

        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=0
            ),
            1,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=1
            ),
            3,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=2
            ),
            9,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=3
            ),
            27,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=0, bracket_iteration=4
            ),
            81,
        )

        # Iteration == 1
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=3, bracket_iteration=0), 3
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=3, bracket_iteration=1), 9
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=3, bracket_iteration=2), 27
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=3, bracket_iteration=3), 81
        )

        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=1, bracket_iteration=0
            ),
            3,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=1, bracket_iteration=1
            ),
            9,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=1, bracket_iteration=2
            ),
            27,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=1, bracket_iteration=3
            ),
            81,
        )

        # Iteration == 2
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=9, bracket_iteration=0), 9
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=9, bracket_iteration=1), 27
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=9, bracket_iteration=2), 81
        )

        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=2, bracket_iteration=0
            ),
            9,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=2, bracket_iteration=1
            ),
            27,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=2, bracket_iteration=2
            ),
            81,
        )

        # Iteration == 3
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=27, bracket_iteration=0), 27
        )
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=27, bracket_iteration=1), 81
        )

        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=3, bracket_iteration=0
            ),
            27,
        )
        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=3, bracket_iteration=1
            ),
            81,
        )

        # Iteration == 4
        self.almost_equal(
            self.manager2.get_n_resources(n_resources=81, bracket_iteration=0), 81
        )

        self.almost_equal(
            self.manager2.get_n_resources_for_iteration(
                iteration=4, bracket_iteration=0
            ),
            81,
        )

    def test_should_reschedule(self):
        assert (
            self.manager1.config.should_create_iteration(
                iteration=0, bracket_iteration=0
            )
            is False
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=0, bracket_iteration=1
            )
            is False
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=0, bracket_iteration=2
            )
            is True
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=0, bracket_iteration=3
            )
            is True
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=1, bracket_iteration=0
            )
            is False
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=1, bracket_iteration=1
            )
            is True
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=1, bracket_iteration=2
            )
            is True
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=2, bracket_iteration=0
            )
            is False
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=2, bracket_iteration=1
            )
            is False
        )
        assert (
            self.manager1.config.should_create_iteration(
                iteration=5, bracket_iteration=0
            )
            is False
        )

    def test_should_reduce_configs(self):
        assert (
            self.manager1.config.should_reduce_configs(iteration=0, bracket_iteration=0)
            is True
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=0, bracket_iteration=1)
            is True
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=0, bracket_iteration=2)
            is False
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=0, bracket_iteration=3)
            is False
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=1, bracket_iteration=0)
            is True
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=1, bracket_iteration=1)
            is False
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=1, bracket_iteration=2)
            is False
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=2, bracket_iteration=0)
            is True
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=2, bracket_iteration=1)
            is False
        )
        assert (
            self.manager1.config.should_reduce_configs(iteration=5, bracket_iteration=0)
            is False
        )

    def test_get_suggestions(self):
        suggestions = self.manager1.get_suggestions(iteration=0, bracket_iteration=2)

        assert len(suggestions) == 9
        for suggestion in suggestions:
            assert "steps" in suggestion
            self.almost_equal(suggestion["steps"], 9.99)
            assert "feature1" in suggestion
            assert "feature2" in suggestion
            assert "feature3" in suggestion

        # Fake iteration
        suggestions = self.manager1.get_suggestions(iteration=1, bracket_iteration=0)
        assert len(suggestions) == 5
        for suggestion in suggestions:
            assert "steps" in suggestion
            self.almost_equal(suggestion["steps"], 3.33)
            assert "feature1" in suggestion
            assert "feature2" in suggestion
            assert "feature3" in suggestion

        # Fake iteration
        suggestions = self.manager1.get_suggestions(iteration=2, bracket_iteration=0)
        assert len(suggestions) == 3
        for suggestion in suggestions:
            assert "steps" in suggestion
            self.almost_equal(suggestion["steps"], 9.99)
            assert "feature1" in suggestion
            assert "feature2" in suggestion
            assert "feature3" in suggestion

        # Manager2
        # Fake iteration
        suggestions = self.manager2.get_suggestions(iteration=2, bracket_iteration=0)
        assert len(suggestions) == 15
        for suggestion in suggestions:
            assert "size" in suggestion
            self.almost_equal(suggestion["size"], 9)
            assert "feature1" in suggestion
            assert "feature2" in suggestion
            assert "feature3" in suggestion
            assert "feature4" in suggestion

        # Fake iteration
        suggestions = self.manager2.get_suggestions(iteration=4, bracket_iteration=0)
        assert len(suggestions) == 5
        for suggestion in suggestions:
            assert "size" in suggestion
            self.almost_equal(suggestion["size"], 81)
            assert "feature1" in suggestion
            assert "feature2" in suggestion
            assert "feature3" in suggestion
            assert "feature4" in suggestion



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.env_vars.getters import get_project_error_message, get_project_or_local
from polyaxon.exceptions import PolyaxonClientException
from tests.utils import BaseTestCase


class TestProjectEnvVars(BaseTestCase):
    def test_get_project_error_message(self):
        assert get_project_error_message("", "") is not None
        assert get_project_error_message("test", "") is not None
        assert get_project_error_message("", "test") is not None
        assert get_project_error_message("test", "test") is None

    def test_get_project_or_local(self):
        with self.assertRaises(PolyaxonClientException):
            get_project_or_local(None)

        assert get_project_or_local("owner.project") == ("owner", "project")



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import polyaxon_sdk

from polyaxon.utils.date_utils import parse_datetime
from polyaxon.utils.tz_utils import now


class V1Stages(polyaxon_sdk.V1Stages):
    """Stage is the information that represents the current stage of an entity's version.

    You can describe what stage a component or a model version is at.

    The supported stages by Polyaxon.

    Enum:
        TESTING: "testing"
        STAGING: "staging"
        PRODUCTION: "production"
        DISABLED: "disabled"
    """

    pass


class V1Statuses(polyaxon_sdk.V1Statuses):
    """Status is the information that represents the current state of a run.

    By examining a run status and/or the history of its statuses,
    you can learn what stage the run is at, and what stages are left.

    The supported statuses by Polyaxon.

    Enum:
        CREATED: "created"
        ON_SCHEDULE: "on_schedule"
        RESUMING: "resuming"
        WARNING: "warning"
        UNSCHEDULABLE: "unschedulable"
        COMPILED: "compiled"
        QUEUED: "queued"
        SCHEDULED: "scheduled"
        STARTING: "starting"
        RUNNING: "running"
        SUCCEEDED: "succeeded"
        FAILED: "failed"
        UPSTREAM_FAILED: "upstream_failed"
        STOPPING: "stopping"
        STOPPED: "stopped"
        SKIPPED: "skipped"
        RETRYING: "retrying"
        UNKNOWN: "unknown"
    """

    allowable_hook_values = [
        polyaxon_sdk.V1Statuses.FAILED,
        polyaxon_sdk.V1Statuses.STOPPED,
        polyaxon_sdk.V1Statuses.SUCCEEDED,
        polyaxon_sdk.V1Statuses.SKIPPED,
        polyaxon_sdk.V1Statuses.UPSTREAM_FAILED,
        polyaxon_sdk.V1Statuses.DONE,
    ]


class StatusColor:
    """The statuses colors.

    Enum:
        GREEN: #1aaa55
        RED: #aa310f
        BLUE: #2e77aa
        YELLOW: #aa9e4a
        GREY: #485563
    """

    GREEN = "#1aaa55"
    RED = "#aa310f"
    BLUE = "#2e77aa"
    YELLOW = "#aa9e4a"
    GREY = "#485563"

    @classmethod
    def get_color(cls, status: str) -> str:
        if status in [
            V1Statuses.FAILED,
            V1Statuses.STOPPED,
            V1Statuses.UPSTREAM_FAILED,
        ]:
            return cls.RED
        if status == V1Statuses.SUCCEEDED:
            return cls.GREEN

        if status == V1Statuses.SKIPPED:
            return cls.GREY

        if LifeCycle.is_done(status):
            return cls.GREY

        return cls.YELLOW


class LifeCycle:
    """The Run LifeCycle is state machine for status transition."""

    WARNING_VALUES = {V1Statuses.UNSCHEDULABLE, V1Statuses.WARNING}
    SAFE_STOP_VALUES = {
        V1Statuses.CREATED,
        V1Statuses.ON_SCHEDULE,
        V1Statuses.RESUMING,
        V1Statuses.COMPILED,
    }
    PENDING_VALUES = {
        V1Statuses.CREATED,
        V1Statuses.ON_SCHEDULE,
        V1Statuses.RESUMING,
    }
    COMPILABLE_VALUES = {
        V1Statuses.CREATED,
        V1Statuses.ON_SCHEDULE,
        V1Statuses.RESUMING,
        V1Statuses.RETRYING,
    }
    RUNNING_VALUES = {
        V1Statuses.SCHEDULED,
        V1Statuses.STARTING,
        V1Statuses.PROCESSING,
        V1Statuses.RUNNING,
    }
    ON_K8S_VALUES = RUNNING_VALUES | WARNING_VALUES | {V1Statuses.UNKNOWN}
    DONE_VALUES = {
        V1Statuses.FAILED,
        V1Statuses.UPSTREAM_FAILED,
        V1Statuses.STOPPED,
        V1Statuses.SKIPPED,
        V1Statuses.SUCCEEDED,
    }
    DONE_OR_IN_PROGRESS_VALUES = DONE_VALUES | {
        V1Statuses.STOPPING,
    }

    @classmethod
    def can_check_heartbeat(cls, status: str) -> bool:
        """Checks if a run with this status is in a state that allows to check for a heartbeat."""
        return status in cls.RUNNING_VALUES

    @classmethod
    def is_unschedulable(cls, status: str) -> bool:
        """Checks if a run with this status is unschedulable."""
        return status == V1Statuses.UNSCHEDULABLE

    @classmethod
    def is_processing(cls, status: str) -> bool:
        """Checks if a run with this status is processing."""
        return status == V1Statuses.PROCESSING

    @classmethod
    def is_warning(cls, status: str) -> bool:
        """Checks if a run with this status is in a warning status."""
        return status in cls.WARNING_VALUES

    @classmethod
    def is_pending(cls, status: str) -> bool:
        """Checks if a run with this status is in a pending status."""
        return status in cls.PENDING_VALUES

    @classmethod
    def is_compiled(cls, status: str) -> bool:
        """Checks if a run with this status is compiled."""
        return status == V1Statuses.COMPILED

    @classmethod
    def is_compilable(cls, status: str) -> bool:
        """Checks if a run with this status is compilable."""
        return status in cls.COMPILABLE_VALUES

    @classmethod
    def is_queued(cls, status: str) -> bool:
        """Checks if a run with this status is queued."""
        return status == V1Statuses.QUEUED

    @classmethod
    def is_starting(cls, status: str) -> bool:
        """Checks if a run with this status is starting."""
        return status == V1Statuses.STARTING

    @classmethod
    def is_running(cls, status: str) -> bool:
        """Checks if a run with this status is running."""
        return status in cls.RUNNING_VALUES

    @classmethod
    def is_unknown(cls, status: str) -> bool:
        """Checks if a run with this status is in an unknown state."""
        return status == V1Statuses.UNKNOWN

    @classmethod
    def is_safe_stoppable(cls, status: str) -> bool:
        """Checks if a run with this status is an be stopped without operator."""
        return status in cls.SAFE_STOP_VALUES

    @classmethod
    def is_k8s_stoppable(cls, status: str) -> bool:
        """Checks if a run with this status is scheduled on k8s and is stoppable."""
        return status in cls.ON_K8S_VALUES

    @classmethod
    def is_stoppable(cls, status: str) -> bool:
        """Checks if a run with this status is stoppable."""
        return not cls.is_done(status)

    @classmethod
    def is_stopping(cls, status: str) -> bool:
        """Checks if a run with this status is stopping."""
        return status == V1Statuses.STOPPING

    @classmethod
    def is_done(cls, status: str, progressing: bool = False) -> bool:
        """Checks if a run with this status is done."""
        if progressing:
            return status in cls.DONE_OR_IN_PROGRESS_VALUES

        return status in cls.DONE_VALUES

    @classmethod
    def failed(cls, status: str) -> bool:
        """Checks if a run with this status is failed."""
        return status == V1Statuses.FAILED or status == V1Statuses.UPSTREAM_FAILED

    @classmethod
    def succeeded(cls, status: str) -> bool:
        """Checks if a run with this status is succeeded."""
        return status == V1Statuses.SUCCEEDED

    @classmethod
    def stopped(cls, status: str) -> bool:
        """Checks if a run with this status is stopped."""
        return status == V1Statuses.STOPPED

    @classmethod
    def skipped(cls, status: str) -> bool:
        """Checks if a run with this status is skipped."""
        return status == V1Statuses.SKIPPED

    @classmethod
    def set_started_at(cls, entity) -> bool:
        # We allow to override started_at if the value is running
        if entity.started_at is not None:
            return False

        if cls.is_running(entity.status):
            entity.started_at = now()
            # Update wait_time
            if entity.wait_time is None:
                entity.wait_time = (entity.started_at - entity.created_at).seconds
            return True

        return False

    @classmethod
    def set_finished_at(cls, entity) -> bool:
        if cls.is_done(entity.status) and entity.finished_at is None:
            entity.finished_at = now()
            if entity.started_at is None:  # We should not have this case
                entity.started_at = entity.created_at
            # Update duration
            if entity.duration is None:
                entity.duration = (entity.finished_at - entity.started_at).seconds
            return True
        return False


class ConditionMixin:
    @classmethod
    def get_condition(
        cls,
        type=None,  # noqa
        status=None,
        last_update_time=None,
        last_transition_time=None,
        reason=None,
        message=None,
    ):
        current_time = now()
        last_update_time = last_update_time or current_time
        last_transition_time = last_transition_time or current_time
        return cls(
            type=type.lower() if type else type,
            status=status,
            last_update_time=last_update_time,
            last_transition_time=last_transition_time,
            reason=reason,
            message=message,
        )

    def __eq__(self, other):
        return self.type == other.type and self.status == other.status

    @classmethod
    def get_last_update_time(cls, value):
        return parse_datetime(value)


class V1StatusCondition(ConditionMixin, polyaxon_sdk.V1StatusCondition):
    pass


class V1StageCondition(ConditionMixin, polyaxon_sdk.V1StageCondition):
    pass



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.utils.string_utils import is_protected_type, slugify
from tests.utils import BaseTestCase


class TestUtils(BaseTestCase):
    def test_is_protected_type(self):
        assert is_protected_type(None) is True
        assert is_protected_type(1) is True
        assert is_protected_type(1.1) is True
        assert is_protected_type("foo") is False


class TestSlugify(BaseTestCase):
    def test_slugify(self):
        self.assertEqual(
            slugify(
                " Jack & Jill like numbers 1,2,3 and 4 and silly characters ?%.$!/"
            ),
            "Jack-Jill-like-numbers-123-and-4-and-silly-characters-",
        )

    def test_unicode(self):
        self.assertEqual(
            slugify("Un \xe9l\xe9phant \xe0 l'or\xe9e du bois"),
            "Un-elephant-a-loree-du-bois",
        )

    def test_non_string_input(self):
        self.assertEqual(slugify(123), "123")



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from marshmallow import ValidationError

from polyaxon.polyflow import V1EventKind, V1EventTrigger
from tests.utils import BaseTestCase, assert_equal_dict


@pytest.mark.events_mark
class TestEventTriggerConfigs(BaseTestCase):
    def test_events_config(self):
        config_dict = {}

        with self.assertRaises(ValidationError):
            V1EventTrigger.from_dict(config_dict)

        config_dict["ref"] = "test"
        with self.assertRaises(ValidationError):
            V1EventTrigger.from_dict(config_dict)

        # Add kinds
        config_dict["kinds"] = [V1EventKind.RUN_STATUS_DONE]
        config = V1EventTrigger.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Set correct ref
        config_dict["ref"] = "ops.A"
        config = V1EventTrigger.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add another event
        config_dict["kinds"] = [
            V1EventKind.RUN_STATUS_RESUMING,
            V1EventKind.RUN_NEW_ARTIFACTS,
        ]
        V1EventTrigger.from_dict(config_dict)

        # Use run event
        config_dict["ref"] = "runs.0de53b5bf8b04a219d12a39c6b92bcce"
        config = V1EventTrigger.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import io
import os
import re
import shutil
import unicodedata

from typing import Dict, List

from polyaxon.constants import UNKNOWN
from polyaxon.logger import logger
from polyaxon.polyboard.artifacts import V1ArtifactKind
from polyaxon.polyboard.events import (
    LoggedEventSpec,
    V1Event,
    V1EventArtifact,
    V1EventAudio,
    V1EventChart,
    V1EventChartKind,
    V1EventDataframe,
    V1EventHistogram,
    V1EventImage,
    V1EventModel,
    V1EventVideo,
)
from polyaxon.polyboard.events.schemas import V1EventCurve, V1EventCurveKind
from polyaxon.utils.np_utils import calculate_scale_factor, to_np
from polyaxon.utils.path_utils import check_or_create_path, module_type

try:
    import numpy as np
except ImportError:
    np = None


NUMPY_ERROR_MESSAGE = "numpy is required for this tracking operation!"
PIL_ERROR_MESSAGE = "PIL/Pillow is required for this tracking operation!"
MOVIEPY_ERROR_MESSAGE = "moviepy is required for this tracking operation!"
MATPLOTLIB_ERROR_MESSAGE = "matplotlib is required for this tracking operation!"
PLOTLY_ERROR_MESSAGE = "plotly is required for this tracking operation!"
BOKEH_ERROR_MESSAGE = "bokeh is required for this tracking operation!"
SKLEARN_ERROR_MESSAGE = "sklearn is required for this tracking operation!"


def copy_file_path(from_path: str, asset_path: str):
    check_or_create_path(asset_path, is_dir=False)
    shutil.copy(from_path, asset_path)


def copy_dir_path(from_path: str, asset_path: str):
    check_or_create_path(asset_path, is_dir=False)
    shutil.copytree(from_path, asset_path)


def copy_file_or_dir_path(from_path: str, asset_path: str):
    check_or_create_path(asset_path, is_dir=False)
    if os.path.isfile(from_path):
        shutil.copy(from_path, asset_path)
    else:
        shutil.copytree(from_path, asset_path)


def dataframe_path(
    from_path: str,
    asset_path: str,
    content_type: str = None,
    asset_rel_path: str = None,
) -> V1EventDataframe:
    copy_file_path(from_path, asset_path)
    return V1EventDataframe(
        path=asset_rel_path or asset_path, content_type=content_type
    )


def model_path(
    from_path: str,
    asset_path: str,
    framework: str = None,
    spec: Dict = None,
    asset_rel_path: str = None,
) -> V1EventModel:
    copy_file_or_dir_path(from_path, asset_path)
    return V1EventModel(
        path=asset_rel_path or asset_path, framework=framework, spec=spec
    )


def artifact_path(
    from_path: str, asset_path: str, kind: str, asset_rel_path: str = None
) -> V1EventArtifact:
    copy_file_or_dir_path(from_path, asset_path)
    return V1EventArtifact(kind=kind, path=asset_rel_path or asset_path)


def image_path(
    from_path: str, asset_path: str, asset_rel_path: str = None
) -> V1EventImage:
    copy_file_path(from_path, asset_path)
    return V1EventImage(path=asset_rel_path or asset_path)


def video_path(
    from_path: str, asset_path: str, content_type=None, asset_rel_path: str = None
) -> V1EventVideo:
    copy_file_path(from_path, asset_path)
    return V1EventVideo(path=asset_rel_path or asset_path, content_type=content_type)


def audio_path(
    from_path: str, asset_path: str, content_type=None, asset_rel_path: str = None
) -> V1EventAudio:
    copy_file_path(from_path, asset_path)
    return V1EventAudio(path=asset_rel_path or asset_path, content_type=content_type)


def _draw_single_box(
    image,
    xmin,
    ymin,
    xmax,
    ymax,
    display_str,
    color="black",
    color_text="black",
    thickness=2,
):
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    try:
        from PIL import ImageDraw, ImageFont
    except ImportError:
        logger.warning(PIL_ERROR_MESSAGE)
        return UNKNOWN

    font = ImageFont.load_default()
    draw = ImageDraw.Draw(image)
    (left, right, top, bottom) = (xmin, xmax, ymin, ymax)
    draw.line(
        [(left, top), (left, bottom), (right, bottom), (right, top), (left, top)],
        width=thickness,
        fill=color,
    )
    if display_str:
        text_bottom = bottom
        # Reverse list and print from bottom to top.
        text_width, text_height = font.getsize(display_str)
        margin = np.ceil(0.05 * text_height)
        draw.rectangle(
            [
                (left, text_bottom - text_height - 2 * margin),
                (left + text_width, text_bottom),
            ],
            fill=color,
        )
        draw.text(
            (left + margin, text_bottom - text_height - margin),
            display_str,
            fill=color_text,
            font=font,
        )
    return image


def metric(value):
    if isinstance(value, float):
        return value

    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    value = to_np(value)
    assert value.squeeze().ndim == 0, "scalar should be 0D"
    return float(value)


def histogram(values, bins, max_bins=None):
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    values = to_np(values).astype(float)

    if values.size == 0:
        raise ValueError("The input has no element.")
    values = values.reshape(-1)
    counts, limits = np.histogram(values, bins=bins)
    num_bins = len(counts)
    if max_bins is not None and num_bins > max_bins:
        subsampling = num_bins // max_bins
        subsampling_remainder = num_bins % subsampling
        if subsampling_remainder != 0:
            counts = np.pad(
                counts,
                pad_width=[[0, subsampling - subsampling_remainder]],
                mode="constant",
                constant_values=0,
            )
        counts = counts.reshape(-1, subsampling).sum(axis=-1)

    if counts.size == 0:
        logger.warning("Tracking an empty histogram")
        return UNKNOWN

    return V1EventHistogram(values=values, counts=counts)


def np_histogram(values, counts):
    return V1EventHistogram(values=values, counts=counts)


def encoded_image(asset_path: str, data, asset_rel_path: str = None):
    try:
        from PIL import Image
    except ImportError:
        logger.warning(PIL_ERROR_MESSAGE)
        return UNKNOWN

    image_data = Image.open(io.BytesIO(data.encoded_image_string))
    return save_image(
        asset_path=asset_rel_path or asset_path,
        image_data=image_data,
        height=data.height,
        width=data.width,
        colorspace=data.colorspace,
    )


def image(
    asset_path: str, data, rescale=1, dataformats="CHW", asset_rel_path: str = None
):
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    tensor = to_np(data)
    tensor = convert_to_HWC(tensor, dataformats)
    # Do not assume that user passes in values in [0, 255], use data type to detect
    scale_factor = calculate_scale_factor(tensor)
    tensor = tensor.astype(np.float32)
    tensor = (tensor * scale_factor).astype(np.uint8)
    return make_image(
        asset_path, tensor, rescale=rescale, asset_rel_path=asset_rel_path
    )


def image_boxes(
    asset_path: str,
    tensor_image,
    tensor_boxes,
    rescale=1,
    dataformats="CHW",
    asset_rel_path: str = None,
):
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    tensor_image = to_np(tensor_image)
    tensor_image = convert_to_HWC(tensor_image, dataformats)
    tensor_boxes = to_np(tensor_boxes)
    tensor_image = tensor_image.astype(np.float32) * calculate_scale_factor(
        tensor_image
    )
    return make_image(
        asset_path,
        tensor_image.astype(np.uint8),
        rescale=rescale,
        rois=tensor_boxes,
        asset_rel_path=asset_rel_path,
    )


def draw_boxes(disp_image, boxes):
    # xyxy format
    num_boxes = boxes.shape[0]
    list_gt = range(num_boxes)
    for i in list_gt:
        disp_image = _draw_single_box(
            disp_image,
            boxes[i, 0],
            boxes[i, 1],
            boxes[i, 2],
            boxes[i, 3],
            display_str=None,
            color="Red",
        )
    return disp_image


def make_image(
    asset_path: str, tensor, rescale=1, rois=None, asset_rel_path: str = None
):
    try:
        from PIL import Image
    except ImportError:
        logger.warning(PIL_ERROR_MESSAGE)
        return UNKNOWN

    height, width, colorspace = tensor.shape
    scaled_height = int(height * rescale)
    scaled_width = int(width * rescale)
    image_data = Image.fromarray(tensor)
    if rois is not None:
        image_data = draw_boxes(image_data, rois)
    image_data = image_data.resize((scaled_width, scaled_height), Image.ANTIALIAS)

    return save_image(
        asset_path=asset_path,
        image_data=image_data,
        height=height,
        width=width,
        colorspace=colorspace,
        asset_rel_path=asset_rel_path,
    )


def save_image(
    asset_path: str, image_data, height, width, colorspace, asset_rel_path: str = None
):
    check_or_create_path(asset_path, is_dir=False)
    image_data.save(asset_path, format="PNG")
    return V1EventImage(
        height=height,
        width=width,
        colorspace=colorspace,
        path=asset_rel_path or asset_path,
    )


def video(
    asset_path: str, tensor, fps=4, content_type="gif", asset_rel_path: str = None
):
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    tensor = to_np(tensor)
    tensor = prepare_video(tensor)
    # If user passes in uint8, then we don't need to rescale by 255
    scale_factor = calculate_scale_factor(tensor)
    tensor = tensor.astype(np.float32)
    tensor = (tensor * scale_factor).astype(np.uint8)
    return make_video(
        asset_path, tensor, fps, content_type, asset_rel_path=asset_rel_path
    )


def make_video(
    asset_path: str, tensor, fps, content_type="gif", asset_rel_path: str = None
):
    try:
        import moviepy  # noqa: F401
    except ImportError:
        logger.warning(MOVIEPY_ERROR_MESSAGE)
        return UNKNOWN
    try:
        from moviepy import editor as mpy
    except ImportError:
        logger.warning(
            "moviepy is installed, but can't import moviepy.editor.",
            "Some packages could be missing [imageio, requests]",
        )
        return

    t, h, w, c = tensor.shape

    # encode sequence of images into gif string
    clip = mpy.ImageSequenceClip(list(tensor), fps=fps)

    check_or_create_path(asset_path, is_dir=False)

    try:  # older version of moviepy
        if content_type == "gif":
            clip.write_gif(asset_path, verbose=False, progress_bar=False)
        else:
            clip.write_videofile(asset_path, verbose=False, progress_bar=False)
    except TypeError:
        if content_type == "gif":
            clip.write_gif(asset_path, verbose=False)
        else:
            clip.write_videofile(asset_path, verbose=False)

    return V1EventVideo(
        height=h,
        width=w,
        colorspace=c,
        path=asset_rel_path or asset_path,
        content_type=content_type,
    )


def audio(asset_path: str, tensor, sample_rate=44100, asset_rel_path: str = None):
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    tensor = to_np(tensor)
    tensor = tensor.squeeze()
    if abs(tensor).max() > 1:
        print("warning: audio amplitude out of range, auto clipped.")
        tensor = tensor.clip(-1, 1)
    assert tensor.ndim == 1, "input tensor should be 1 dimensional."

    tensor_list = [int(32767.0 * x) for x in tensor]

    import struct
    import wave

    check_or_create_path(asset_path, is_dir=False)

    wave_write = wave.open(asset_path, "wb")
    wave_write.setnchannels(1)
    wave_write.setsampwidth(2)
    wave_write.setframerate(sample_rate)
    tensor_enc = b""
    for v in tensor_list:
        tensor_enc += struct.pack("<h", v)

    wave_write.writeframes(tensor_enc)
    wave_write.close()
    return V1EventAudio(
        sample_rate=sample_rate,
        num_channels=1,
        length_frames=len(tensor_list),
        path=asset_rel_path or asset_path,
        content_type="audio/wav",
    )


def roc_auc_curve(fpr, tpr, auc=None):
    return V1EventCurve(
        kind=V1EventCurveKind.ROC,
        x=fpr,
        y=tpr,
        annotation=str(auc) if auc else None,
    )


def sklearn_roc_auc_curve(y_preds, y_targets):
    try:
        from sklearn.metrics import auc, roc_curve
    except ImportError:
        logger.warning(SKLEARN_ERROR_MESSAGE)

    try:
        y_true = y_targets.numpy()
    except AttributeError:
        y_true = y_targets
    try:
        y_pred = y_preds.numpy()
    except AttributeError:
        y_pred = y_preds
    fpr, tpr, _ = roc_curve(y_true, y_pred)
    auc_score = auc(fpr, tpr)
    return V1EventCurve(
        kind=V1EventCurveKind.ROC,
        x=fpr,
        y=tpr,
        annotation=str(auc_score),
    )


def pr_curve(precision, recall, average_precision=None):
    return V1EventCurve(
        kind=V1EventCurveKind.PR,
        x=precision,
        y=recall,
        annotation=str(average_precision) if average_precision else None,
    )


def sklearn_pr_curve(y_preds, y_targets):
    try:
        from sklearn.metrics import average_precision_score, precision_recall_curve
    except ImportError:
        logger.warning(SKLEARN_ERROR_MESSAGE)

    try:
        y_true = y_targets.numpy()
    except AttributeError:
        y_true = y_targets
    try:
        y_pred = y_preds.numpy()
    except AttributeError:
        y_pred = y_preds

    precision, recall, _ = precision_recall_curve(y_true, y_pred)
    ap = average_precision_score(y_true, y_pred)
    return V1EventCurve(
        kind=V1EventCurveKind.PR,
        x=precision,
        y=recall,
        annotation=str(ap),
    )


def curve(x, y, annotation=None):
    return V1EventCurve(
        kind=V1EventCurveKind.CUSTOM,
        x=x,
        y=y,
        annotation=str(annotation) if annotation else None,
    )


def figure_to_image(figure, close=True):
    """Render matplotlib figure to numpy format.

    Returns:
        numpy.array: image in [CHW] order
    """
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)

    try:
        import matplotlib.backends.backend_agg as plt_backend_agg
        import matplotlib.pyplot as plt
    except ImportError:
        logger.warning(MATPLOTLIB_ERROR_MESSAGE)

    canvas = plt_backend_agg.FigureCanvasAgg(figure)
    canvas.draw()
    data = np.frombuffer(canvas.buffer_rgba(), dtype=np.uint8)
    w, h = figure.canvas.get_width_height()
    image_hwc = data.reshape([h, w, 4])[:, :, 0:3]
    image_chw = np.moveaxis(image_hwc, source=2, destination=0)
    if close:
        try:
            plt.close(figure.number)
        except:
            plt.close(figure)
    return image_chw


def figures_to_images(figures, close=True):
    """Render matplotlib figure to numpy format.

    Returns:
        numpy.array: image in [CHW] order
    """
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    images = [figure_to_image(figure, close=close) for figure in figures]
    return np.stack(images)


def ensure_matplotlib_figure(figure):
    """Extract the current figure from a matplotlib object or return the object if it's a figure.
    raises ValueError if the object can't be converted.
    """
    import matplotlib

    from matplotlib.figure import Figure

    if figure == matplotlib.pyplot:
        figure = figure.gcf()
    elif not isinstance(figure, Figure):
        if hasattr(figure, "figure"):
            figure = figure.figure
            # Some matplotlib objects have a figure function
            if not isinstance(figure, Figure):
                raise ValueError(
                    "Only matplotlib.pyplot or matplotlib.pyplot.Figure objects are accepted."
                )
    if not figure.gca().has_data():
        raise ValueError(
            "You attempted to log an empty plot, "
            "pass a figure directly or ensure the global plot isn't closed."
        )
    return figure


def prepare_video(data):
    """
    Converts a 5D tensor [batchsize, time(frame), channel(color), height, width]
    into 4D tensor with dimension [time(frame), new_width, new_height, channel].
    A batch of images are spreaded to a grid, which forms a frame.
    e.g. Video with batchsize 16 will have a 4x4 grid.
    """
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    b, t, c, h, w = data.shape

    if data.dtype == np.uint8:
        data = np.float32(data) / 255.0

    def is_power2(num):
        return num != 0 and ((num & (num - 1)) == 0)

    # pad to nearest power of 2, all at once
    if not is_power2(data.shape[0]):
        len_addition = int(2 ** data.shape[0].bit_length() - data.shape[0])
        data = np.concatenate(
            (data, np.zeros(shape=(len_addition, t, c, h, w))), axis=0
        )

    n_rows = 2 ** ((b.bit_length() - 1) // 2)
    n_cols = data.shape[0] // n_rows

    data = np.reshape(data, newshape=(n_rows, n_cols, t, c, h, w))
    data = np.transpose(data, axes=(2, 0, 4, 1, 5, 3))
    return np.reshape(data, newshape=(t, n_rows * h, n_cols * w, c))


def make_grid(data, ncols=8):
    # I: N1HW or N3HW
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    assert isinstance(data, np.ndarray), "plugin error, should pass numpy array here"
    if data.shape[1] == 1:
        data = np.concatenate([data, data, data], 1)
    assert data.ndim == 4 and data.shape[1] == 3 or data.shape[1] == 4
    nimg = data.shape[0]
    H = data.shape[2]  # noqa
    W = data.shape[3]  # noqa
    ncols = min(nimg, ncols)
    nrows = int(np.ceil(float(nimg) / ncols))
    canvas = np.zeros((data.shape[1], H * nrows, W * ncols))
    i = 0
    for y in range(nrows):
        for x in range(ncols):
            if i >= nimg:
                break
            canvas[:, y * H : (y + 1) * H, x * W : (x + 1) * W] = data[i]  # noqa
            i = i + 1
    return canvas


def convert_to_HWC(tensor, input_format):  # noqa
    if not np:
        logger.warning(NUMPY_ERROR_MESSAGE)
        return UNKNOWN

    assert len(set(input_format)) == len(
        input_format
    ), "You can not use the same dimension shordhand twice. \
        input_format: {}".format(
        input_format
    )
    assert len(tensor.shape) == len(
        input_format
    ), "size of input tensor and input format are different. \
        tensor shape: {}, input_format: {}".format(
        tensor.shape, input_format
    )
    input_format = input_format.upper()

    if len(input_format) == 4:
        index = [input_format.find(c) for c in "NCHW"]
        tensor_NCHW = tensor.transpose(index)  # noqa
        tensor_CHW = make_grid(tensor_NCHW)  # noqa
        return tensor_CHW.transpose(1, 2, 0)

    if len(input_format) == 3:
        index = [input_format.find(c) for c in "HWC"]
        tensor_HWC = tensor.transpose(index)  # noqa
        if tensor_HWC.shape[2] == 1:
            tensor_HWC = np.concatenate([tensor_HWC, tensor_HWC, tensor_HWC], 2)  # noqa
        return tensor_HWC

    if len(input_format) == 2:
        index = [input_format.find(c) for c in "HW"]
        tensor = tensor.transpose(index)
        tensor = np.stack([tensor, tensor, tensor], 2)
        return tensor


def bokeh_chart(figure) -> V1EventChart:
    try:
        from bokeh.embed import json_item
    except ImportError:
        logger.warning(BOKEH_ERROR_MESSAGE)
        return UNKNOWN
    return V1EventChart(kind=V1EventChartKind.BOKEH, figure=json_item(figure))


def altair_chart(figure) -> V1EventChart:
    return V1EventChart(kind=V1EventChartKind.VEGA, figure=figure.to_dict())


def plotly_chart(figure) -> V1EventChart:
    try:
        import plotly.tools
    except ImportError:
        logger.warning(PLOTLY_ERROR_MESSAGE)
        return UNKNOWN

    if module_type(figure, "matplotlib.figure.Figure"):
        figure = plotly.tools.mpl_to_plotly(figure)

    else:
        figure = plotly.tools.return_figure_from_figure_or_data(
            figure, validate_figure=True
        )
    return V1EventChart(kind=V1EventChartKind.PLOTLY, figure=figure)


def mpl_plotly_chart(figure) -> V1EventChart:
    try:
        import plotly.tools
    except ImportError:
        logger.warning(PLOTLY_ERROR_MESSAGE)
        return UNKNOWN

    try:
        import matplotlib

        from matplotlib.figure import Figure
    except ImportError:
        logger.warning(MATPLOTLIB_ERROR_MESSAGE)

    if module_type(figure, "matplotlib.figure.Figure"):
        pass
    else:
        if figure == matplotlib.pyplot:
            figure = figure.gcf()
        elif not isinstance(figure, Figure):
            if hasattr(figure, "figure"):
                figure = figure.figure
                # Some matplotlib objects have a figure function
                if not isinstance(figure, Figure):
                    raise ValueError(
                        "Only matplotlib.pyplot or matplotlib.pyplot.Figure objects are accepted."
                    )

    # This code was taken from:
    # https://github.com/matplotlib/matplotlib/pull/16772/files#diff-506cc6d736a0593e8bb820981b2c12ae # noqa
    # Removed in https://github.com/matplotlib/matplotlib/pull/16772
    from matplotlib.spines import Spine

    def is_frame_like(self):
        """return True if directly on axes frame
        This is useful for determining if a spine is the edge of an
        old style MPL plot. If so, this function will return True.
        """
        self._ensure_position_is_set()
        position = self._position
        if isinstance(position, str):
            if position == "center":
                position = ("axes", 0.5)
            elif position == "zero":
                position = ("data", 0)
        if len(position) != 2:
            raise ValueError("position should be 2-tuple")
        position_type, amount = position
        if position_type == "outward" and amount == 0:
            return True
        else:
            return False

    Spine.is_frame_like = is_frame_like
    figure = plotly.tools.mpl_to_plotly(figure)
    return plotly_chart(figure=figure)


def metrics_dict_to_list(metrics: Dict) -> List:
    results = []
    for k, v in metrics.items():
        results.append(
            LoggedEventSpec(
                name=k,
                kind=V1ArtifactKind.METRIC,
                event=V1Event.make(metric=v),
            )
        )
    return results


def to_fqn_event(name: str) -> str:
    if not name:
        raise ValueError("A name is required to process events.")

    value = str(name)
    value = (
        unicodedata.normalize("NFKD", value).encode("ascii", "ignore").decode("ascii")
    )
    value = re.sub(r"[^\w\\\/\.\s-]", "", value).strip()
    value = re.sub(r"[\\\/]+", "__", value)
    value = re.sub(r"[-\.\s]+", "-", value)
    return value



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import polyaxon_sdk

from marshmallow import fields, validate

from polyaxon.polyflow.early_stopping import EarlyStoppingSchema
from polyaxon.polyflow.matrix.base import BaseSearchConfig
from polyaxon.polyflow.matrix.kinds import V1MatrixKind
from polyaxon.schemas.base import BaseCamelSchema
from polyaxon.schemas.fields.ref_or_obj import RefOrObject


class MappingSchema(BaseCamelSchema):
    kind = fields.Str(allow_none=True, validate=validate.Equal(V1MatrixKind.MAPPING))
    values = RefOrObject(fields.List(fields.Dict(), required=True), required=True)
    concurrency = RefOrObject(fields.Int(allow_none=True))
    early_stopping = fields.List(fields.Nested(EarlyStoppingSchema), allow_none=True)

    @staticmethod
    def schema_config():
        return V1Mapping


class V1Mapping(BaseSearchConfig, polyaxon_sdk.V1Mapping):
    """Mapping is a flexible way for dynamically executing a component sequentially or in parallel
    based on a list of parameter combinations.

    Args:
        kind: str, should be equal `mapping`
        values: List[Dict]
        concurrency: int, optional
        early_stopping: List[[EarlyStopping](/docs/automation/helpers/early-stopping)], optional

    ## YAML usage

    ```yaml
    >>> matrix:
    >>>   kind: mapping
    >>>   values:
    >>>   concurrency:
    >>>   earlyStopping:
    ```

    ## Python usage

    ```python
    >>> from polyaxon.polyflow import V1Mapping, V1FailureEarlyStopping
    >>> mapping = V1Mapping(
    >>>     values=[{...}, {...}, ...],
    >>>     concurrency=4,
    >>>     early_stopping=[V1FailureEarlyStopping(...)]
    >>> )
    ```

    ## Fields

    ### kind

    The kind signals to the CLI, client, and other tools that this matrix is mapping.

    If you are using the python client to create the mapping,
    this field is not required and is set by default.

    ```yaml
    >>> matrix:
    >>>   kind: mapping
    ```

    ### values

    A List of dictionaries (key/value objects) that will be used to pass
    those dictionaries as params to each execution.

    ```yaml
    >>> matrix:
    >>>   values:
    >>>     - lr: 0.001
    >>>       dropout: 0.1
    >>>     - lr: 0.01
    >>>       dropout: 0.2
    >>>     - lr: 0.1
    >>>       dropout: 0.3
    ```

    ### concurrency

    An optional value to set the number of concurrent operations.

    <blockquote class="light">
    This value only makes sense if less or equal to the total number of possible runs.
    </blockquote>

    ```yaml
    >>> matrix:
    >>>   concurrency: 2
    ```

    For more details about concurrency management,
    please check the [concurrency section](/docs/automation/helpers/concurrency/).

    ### earlyStopping

    A list of early stopping conditions to check for terminating
    all operations managed by the pipeline.
    If one of the early stopping conditions is met,
    a signal will be sent to terminate all running and pending operations.

    ```yaml
    >>> matrix:
    >>>   earlyStopping: ...
    ```

    For more details please check the
    [early stopping section](/docs/automation/helpers/early-stopping/).
    """

    SCHEMA = MappingSchema
    IDENTIFIER = V1MatrixKind.MAPPING
    REDUCED_ATTRIBUTES = ["concurrency", "earlyStopping"]

    def has_key(self, key: str):
        return self.values and key in set(self.values[0].keys())



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from polyaxon import settings
from polyaxon.api import (
    ADMIN_V1_LOCATION,
    API_V1_LOCATION,
    AUTH_V1_LOCATION,
    HEALTHZ_LOCATION,
    SSO_V1_LOCATION,
    UI_V1_LOCATION,
)
from polyaxon.proxies.schemas.base import clean_config, get_config

UWSGI_OPTIONS = """
location {path} {{
    include     /etc/nginx/uwsgi_params;
    uwsgi_pass  polyaxon;
    uwsgi_param Host				$host;
    uwsgi_param X-Real-IP			$remote_addr;
    uwsgi_param X-Forwarded-For		$proxy_add_x_forwarded_for;
    uwsgi_param X-Forwarded-Proto	$http_x_forwarded_proto;
    uwsgi_intercept_errors {intercept_errors};
}}
"""


def get_api_config(path: str, intercept_errors: str = "on"):
    return get_config(
        options=UWSGI_OPTIONS,
        indent=0,
        path=path,
        intercept_errors=intercept_errors,
    )


def get_uwsgi_config():
    config = [
        get_api_config(path="= /"),
        get_api_config(path="= {}".format(HEALTHZ_LOCATION), intercept_errors="off"),
        get_api_config(path=API_V1_LOCATION, intercept_errors="off"),
        get_api_config(path=AUTH_V1_LOCATION, intercept_errors="off"),
        get_api_config(path=SSO_V1_LOCATION, intercept_errors="off"),
        get_api_config(path=UI_V1_LOCATION),
    ]
    if settings.PROXIES_CONFIG.ui_admin_enabled:
        config.append(get_api_config(path=ADMIN_V1_LOCATION))
    return clean_config(config)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json

from collections import namedtuple
from typing import Dict, Mapping, Optional, Union

import polyaxon_sdk

from marshmallow import ValidationError, fields, pre_load, validate, validates_schema

from polyaxon.parser import parser
from polyaxon.polyboard.artifacts.kinds import V1ArtifactKind
from polyaxon.polyboard.utils import validate_csv
from polyaxon.schemas.base import BaseConfig, BaseSchema
from polyaxon.utils.date_utils import parse_datetime
from polyaxon.utils.np_utils import sanitize_np_types
from polyaxon.utils.signal_decorators import check_partial
from polyaxon.utils.tz_utils import now


class EventImageSchema(BaseSchema):
    height = fields.Int(allow_none=True)
    width = fields.Int(allow_none=True)
    colorspace = fields.Int(allow_none=True)
    path = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventImage


class V1EventImage(BaseConfig, polyaxon_sdk.V1EventImage):
    IDENTIFIER = "image"
    SCHEMA = EventImageSchema
    REDUCED_ATTRIBUTES = ["height", "width", "colorspace", "path"]


class EventVideoSchema(BaseSchema):
    height = fields.Int(allow_none=True)
    width = fields.Int(allow_none=True)
    colorspace = fields.Int(allow_none=True)
    path = fields.Str(allow_none=True)
    content_type = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventVideo


class V1EventVideo(BaseConfig, polyaxon_sdk.V1EventVideo):
    IDENTIFIER = "video"
    SCHEMA = EventImageSchema
    REDUCED_ATTRIBUTES = ["height", "width", "colorspace", "path", "content_type"]


class EventDataframeSchema(BaseSchema):
    path = fields.Str(allow_none=True)
    content_type = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventDataframe


class V1EventDataframe(BaseConfig, polyaxon_sdk.V1EventDataframe):
    IDENTIFIER = "dataframe"
    SCHEMA = EventDataframeSchema
    REDUCED_ATTRIBUTES = ["path", "content_type"]


class EventHistogramSchema(BaseSchema):
    values = fields.List(fields.Float(), allow_none=True)
    counts = fields.List(fields.Float(), allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventHistogram


class V1EventHistogram(BaseConfig, polyaxon_sdk.V1EventHistogram):
    IDENTIFIER = "histogram"
    SCHEMA = EventHistogramSchema
    REDUCED_ATTRIBUTES = ["values", "counts"]


class EventAudioSchema(BaseSchema):
    sample_rate = fields.Float(allow_none=True)
    num_channels = fields.Int(allow_none=True)
    length_frames = fields.Int(allow_none=True)
    path = fields.Str(allow_none=True)
    content_type = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventAudio


class V1EventAudio(BaseConfig, polyaxon_sdk.V1EventAudio):
    IDENTIFIER = "audio"
    SCHEMA = EventAudioSchema
    REDUCED_ATTRIBUTES = [
        "sample_rate",
        "num_channels",
        "length_frames",
        "path",
        "content_type",
    ]


class V1EventChartKind(polyaxon_sdk.V1EventChartKind):
    pass


class EventChartSchema(BaseSchema):
    kind = fields.Str(
        allow_none=True, validate=validate.OneOf(V1EventChartKind.allowable_values)
    )
    figure = fields.Dict(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventChart


class V1EventChart(BaseConfig, polyaxon_sdk.V1EventChart):
    IDENTIFIER = "chart"
    SCHEMA = EventChartSchema
    REDUCED_ATTRIBUTES = ["kind", "figure"]

    def to_dict(self, humanize_values=False, unknown=None, dump=False):
        if self.kind == V1EventChartKind.PLOTLY:
            import plotly.tools

            obj = self.obj_to_dict(
                self, humanize_values=humanize_values, unknown=unknown
            )
            return json.dumps(obj, cls=plotly.utils.PlotlyJSONEncoder)
        # Resume normal serialization
        return super().to_dict(humanize_values, unknown, dump)


class V1EventCurveKind(polyaxon_sdk.V1EventCurveKind):
    pass


class EventCurveSchema(BaseSchema):
    kind = fields.Str(
        allow_none=True, validate=validate.OneOf(V1EventCurveKind.allowable_values)
    )
    x = fields.List(fields.Float(), allow_none=True)
    y = fields.List(fields.Float(), allow_none=True)
    annotation = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventCurve


class V1EventCurve(BaseConfig, polyaxon_sdk.V1EventCurve):
    IDENTIFIER = "curve"
    SCHEMA = EventCurveSchema
    REDUCED_ATTRIBUTES = ["kind", "x", "y", "annotation"]


class EventArtifactSchema(BaseSchema):
    kind = fields.Str(
        allow_none=True, validate=validate.OneOf(V1ArtifactKind.allowable_values)
    )
    path = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventArtifact


class V1EventArtifact(BaseConfig, polyaxon_sdk.V1EventArtifact):
    IDENTIFIER = "artifact"
    SCHEMA = EventArtifactSchema
    REDUCED_ATTRIBUTES = ["kind", "path"]


class EventModelSchema(BaseSchema):
    framework = fields.Str(allow_none=True)
    path = fields.Str(allow_none=True)
    spec = fields.Raw(allow_none=True)

    @staticmethod
    def schema_config():
        return V1EventModel


class V1EventModel(BaseConfig, polyaxon_sdk.V1EventModel):
    IDENTIFIER = "artifact"
    SCHEMA = EventModelSchema
    REDUCED_ATTRIBUTES = ["framework", "path", "spec"]


class EventSchema(BaseSchema):
    timestamp = fields.DateTime(allow_none=True)
    step = fields.Int(allow_none=True)
    metric = fields.Float(allow_none=True)
    image = fields.Nested(EventImageSchema, allow_none=True)
    histogram = fields.Nested(EventHistogramSchema, allow_none=True)
    audio = fields.Nested(EventAudioSchema, allow_none=True)
    video = fields.Nested(EventVideoSchema, allow_none=True)
    html = fields.Str(allow_none=True)
    text = fields.Str(allow_none=True)
    chart = fields.Nested(EventChartSchema, allow_none=True)
    curve = fields.Nested(EventCurveSchema, allow_none=True)
    artifact = fields.Nested(EventArtifactSchema, allow_none=True)
    model = fields.Nested(EventModelSchema, allow_none=True)
    dataframe = fields.Nested(EventDataframeSchema, allow_none=True)

    @staticmethod
    def schema_config():
        return V1Event

    @pre_load
    def pre_validate(self, data, **kwargs):
        if data.get("image") is not None:
            data["image"] = parser.get_dict(
                key="image",
                value=data["image"],
            )
        if data.get("histogram") is not None:
            data["histogram"] = parser.get_dict(
                key="histogram",
                value=data["histogram"],
            )
        if data.get("audio") is not None:
            data["audio"] = parser.get_dict(
                key="audio",
                value=data["audio"],
            )
        if data.get("video") is not None:
            data["video"] = parser.get_dict(
                key="video",
                value=data["video"],
            )
        if data.get("chart") is not None:
            data["chart"] = parser.get_dict(
                key="chart",
                value=data["chart"],
            )
        if data.get("curve") is not None:
            data["curve"] = parser.get_dict(
                key="curve",
                value=data["curve"],
            )
        if data.get("artifact") is not None:
            data["artifact"] = parser.get_dict(
                key="artifact",
                value=data["artifact"],
            )
        if data.get("model") is not None:
            data["model"] = parser.get_dict(
                key="model",
                value=data["model"],
            )
        if data.get("dataframe") is not None:
            data["dataframe"] = parser.get_dict(
                key="dataframe",
                value=data["dataframe"],
            )

        return data

    @validates_schema
    @check_partial
    def validate_event(self, values, **kwargs):
        count = 0

        def increment(c):
            c += 1
            if c > 1:
                raise ValidationError(
                    "An event should have one and only one primitive, found {}.".format(
                        c
                    )
                )
            return c

        if values.get("metric") is not None:
            count = increment(count)
        if values.get("image") is not None:
            count = increment(count)
        if values.get("histogram") is not None:
            count = increment(count)
        if values.get("audio") is not None:
            count = increment(count)
        if values.get("video") is not None:
            count = increment(count)
        if values.get("html") is not None:
            count = increment(count)
        if values.get("text") is not None:
            count = increment(count)
        if values.get("chart") is not None:
            count = increment(count)
        if values.get("curve") is not None:
            count = increment(count)
        if values.get("artifact") is not None:
            count = increment(count)
        if values.get("model") is not None:
            count = increment(count)
        if values.get("dataframe") is not None:
            count = increment(count)

        if count != 1:
            raise ValidationError(
                "An event should have one and only one primitive, found {}.".format(
                    count
                )
            )


class V1Event(BaseConfig, polyaxon_sdk.V1Event):
    SEPARATOR = "|"
    IDENTIFIER = "event"
    SCHEMA = EventSchema
    REDUCED_ATTRIBUTES = [
        "metric",
        "image",
        "histogram",
        "audio",
        "video",
        "html",
        "text",
        "chart",
        "curve",
        "artifact",
        "model",
        "dataframe",
    ]

    @classmethod
    def make(
        cls,
        step: int = None,
        timestamp=None,
        metric: float = None,
        image: V1EventImage = None,
        histogram: V1EventHistogram = None,
        audio: V1EventAudio = None,
        video: V1EventVideo = None,
        html: str = None,
        text: str = None,
        chart: V1EventChart = None,
        curve: V1EventCurve = None,
        artifact: V1EventArtifact = None,
        model: V1EventModel = None,
        dataframe: V1EventDataframe = None,
    ) -> "V1Event":
        if isinstance(timestamp, str):
            try:
                timestamp = parse_datetime(timestamp)
            except Exception as e:
                raise ValidationError("Received an invalid timestamp") from e

        return cls(
            timestamp=timestamp if timestamp else now(tzinfo=True),
            step=step,
            metric=metric,
            image=image,
            histogram=histogram,
            audio=audio,
            video=video,
            html=html,
            text=text,
            chart=chart,
            curve=curve,
            artifact=artifact,
            model=model,
            dataframe=dataframe,
        )

    def get_value(self, dump=True):
        if self.metric is not None:
            return str(self.metric) if dump else self.metric
        if self.image is not None:
            return self.image.to_dict(dump=dump) if dump else self.image
        if self.histogram is not None:
            return self.histogram.to_dict(dump=dump) if dump else self.histogram
        if self.audio is not None:
            return self.audio.to_dict(dump=dump) if dump else self.audio
        if self.video is not None:
            return self.video.to_dict(dump=dump) if dump else self.video
        if self.html is not None:
            return self.html
        if self.text is not None:
            return self.text
        if self.chart is not None:
            return self.chart.to_dict(dump=dump) if dump else self.chart
        if self.curve is not None:
            return self.curve.to_dict(dump=dump) if dump else self.curve
        if self.artifact is not None:
            return self.artifact.to_dict(dump=dump) if dump else self.artifact
        if self.model is not None:
            return self.model.to_dict(dump=dump) if dump else self.model
        if self.dataframe is not None:
            return self.dataframe.to_dict(dump=dump) if dump else self.dataframe

    def to_csv(self) -> str:
        values = [
            str(self.step) if self.step is not None else "",
            str(self.timestamp) if self.timestamp is not None else "",
            self.get_value(dump=True),
        ]

        return self.SEPARATOR.join(values)


class V1Events:
    ORIENT_CSV = "csv"
    ORIENT_DICT = "dict"

    def __init__(self, kind, name, df):
        self.kind = kind
        self.name = name
        self.df = df

    @classmethod
    def read(
        cls, kind: str, name: str, data: Union[str, Dict], parse_dates: bool = True
    ) -> "V1Events":
        import pandas as pd

        if isinstance(data, str):
            csv = validate_csv(data)
            if parse_dates:
                df = pd.read_csv(
                    csv,
                    sep=V1Event.SEPARATOR,
                    parse_dates=["timestamp"],
                )
            else:
                df = pd.read_csv(
                    csv,
                    sep=V1Event.SEPARATOR,
                )
        elif isinstance(data, dict):
            df = pd.DataFrame.from_dict(data)
        else:
            raise ValueError(
                "V1Events received an unsupported value type: {}".format(type(data))
            )

        return cls(name=name, kind=kind, df=df)

    def to_dict(self, orient: str = "list") -> Dict:
        import numpy as np

        return self.df.replace({np.nan: None}).to_dict(orient=orient)

    def get_event_at(self, index):
        event = self.df.iloc[index].to_dict()
        event["timestamp"] = event["timestamp"].isoformat()
        event["step"] = sanitize_np_types(event["step"])
        return V1Event.from_dict(event)

    def _get_step_summary(self) -> Optional[Dict]:
        _count = self.df.step.count()
        if _count == 0:
            return None

        return {
            "count": sanitize_np_types(_count),
            "min": sanitize_np_types(self.df.step.iloc[0]),
            "max": sanitize_np_types(self.df.step.iloc[-1]),
        }

    def _get_ts_summary(self) -> Optional[Dict]:
        _count = self.df.timestamp.count()
        if _count == 0:
            return None

        return {
            "min": self.df.timestamp.iloc[0].isoformat(),
            "max": self.df.timestamp.iloc[-1].isoformat(),
        }

    def get_summary(self) -> Dict:
        summary = {}
        step_summary = self._get_step_summary()
        if step_summary:
            summary["step"] = step_summary

        ts_summary = self._get_ts_summary()
        if ts_summary:
            summary["timestamp"] = ts_summary

        if self.kind == V1ArtifactKind.METRIC:
            summary[self.kind] = {
                k: sanitize_np_types(v)
                for k, v in self.df.metric.describe().to_dict().items()
            }
            summary[self.kind]["last"] = sanitize_np_types(self.df.metric.iloc[-1])

        return summary


class LoggedEventSpec(namedtuple("LoggedEventSpec", "name kind event")):
    pass


class LoggedEventListSpec(namedtuple("LoggedEventListSpec", "name kind events")):
    def get_csv_header(self) -> str:
        return V1Event.SEPARATOR.join(["step", "timestamp", self.kind])

    def get_csv_events(self) -> str:
        events = ["\n{}".format(e.to_csv()) for e in self.events]
        return "".join(events)

    def empty_events(self):
        self.events[:] = []

    def to_dict(self):
        return {
            "name": self.name,
            "kind": self.kind,
            "events": [e.to_dict() for e in self.events],
        }

    @classmethod
    def from_dict(cls, value: Mapping) -> "LoggedEventListSpec":
        return cls(
            name=value.get("name"),
            kind=value.get("kind"),
            events=[V1Event.from_dict(e) for e in value.get("events", [])],
        )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import click

from polyaxon_sdk import V1ModelRegistry, V1ModelVersion
from polyaxon_sdk.rest import ApiException
from urllib3.exceptions import HTTPError

from polyaxon.cli.dashboard import get_dashboard_url
from polyaxon.cli.errors import handle_cli_error
from polyaxon.cli.options import (
    OPTIONS_MODEL_REGISTRY,
    OPTIONS_MODEL_VERSION,
    OPTIONS_OWNER,
)
from polyaxon.cli.utils import get_entity_details
from polyaxon.client import PolyaxonClient
from polyaxon.env_vars.getters import get_model_info
from polyaxon.env_vars.getters.user import get_local_owner
from polyaxon.exceptions import PolyaxonException
from polyaxon.logger import clean_outputs
from polyaxon.polyaxonfile import get_specification
from polyaxon.utils.formatting import (
    Printer,
    dict_tabulate,
    dict_to_tabulate,
    get_meta_response,
    list_dicts_to_tabulate,
)
from polyaxon.utils.query_params import get_query_params
from polyaxon.utils.validation import validate_tags


def get_model_version_details(response):
    metadata = response.metadata
    response = dict_to_tabulate(
        response.to_dict(), humanize_values=True, exclude_attrs=["metadata"]
    )

    Printer.print_header("Model version info:")
    dict_tabulate(response)

    if metadata:
        Printer.print_header("Metadata:")
        click.echo(metadata)


def get_info(model: str = None, version: str = None):
    if not any([model, version]):
        Printer.print_error(
            "A model registry or a model version is required.", sys_exit=True
        )
    if all([model, version]):
        Printer.print_error(
            "Only a model registry or a model version is required, not both.",
            sys_exit=True,
        )

    if model:
        entity = model
        entity_name = "model registry"
        is_version = False
    else:
        entity = version
        entity_name = "model version"
        is_version = True

    try:
        owner, model_registry, model_version = get_model_info(
            entity=entity, entity_name=entity_name, is_cli=True
        )
        return owner, model_registry, model_version, is_version
    except PolyaxonException as e:
        handle_cli_error(
            e,
            message="Could not resolve the {} from the value `{}`.".format(
                entity_name, entity
            ),
            sys_exit=True,
        )


@click.group()
@clean_outputs
def registry():
    """Commands for model registry."""


@registry.command()
@click.option(
    "--name",
    type=str,
    help="The model registry name, e.g. 'model-A' or 'acme/model-A:latest'.",
)
@click.option("--description", type=str, help="Description of the model.")
@click.option("--tags", type=str, help="Tags of the model, comma separated values.")
@click.option(
    "--public", is_flag=True, help="Set the visibility of the model to public."
)
@clean_outputs
def create(name, description, tags, public):
    """Create a new model.

    Example:

    \b
    $ polyaxon registry create --name=kaniko --description="Tool to build container images"

    \b
    $ polyaxon registry create --name=owner/name --description="Model description"
    """
    if not name:
        Printer.print_error(
            "Please provide a name to create a model registry.",
            command_help="registry create",
            sys_exit=True,
        )
    owner, registry_name, _, _ = get_info(name, None)

    tags = validate_tags(tags)

    if not owner or not registry_name:
        Printer.print_error(
            "Please provide a valid model name with --name=owner/registry-name. "
        )
        sys.exit(1)

    try:
        registry_config = V1ModelRegistry(
            name=registry_name, description=description, tags=tags, is_public=public
        )
        polyaxon_client = PolyaxonClient()
        _registry = polyaxon_client.model_registry_v1.create_model_registry(
            owner, registry_config
        )
    except (ApiException, HTTPError) as e:
        handle_cli_error(
            e, message="Could not create model registry `{}`.".format(registry_name)
        )
        sys.exit(1)

    Printer.print_success(
        "Model registry `{}` was created successfully.".format(_registry.name)
    )
    click.echo(
        "You can view this model registry on Polyaxon UI: {}".format(
            get_dashboard_url(subpath="{}/registry/{}".format(owner, _registry.name))
        )
    )


@registry.command()
@click.option(
    "--name",
    type=str,
    help="The model version name, e.g. 'kaniko' or 'kaniko:1.2' "
    "or 'acme/kaniko:latest' or 'acme/kaniko:dev'.",
)
@click.option("--description", type=str, help="Description of the version.")
@click.option("--tags", type=str, help="Tags of the version, comma separated values.")
@click.option(
    "--run-uid", "-uid", type=str, help="The run to link to this model version."
)
@clean_outputs
def push(name, description, tags, run_uid):
    """Push a new model version.
    If the name corresponds to an existing model version, it will be updated.

    Example:

    \b
    $ polyaxon registry push -f polyaxonfile.yaml --name=kaniko:latest --description="Tool to build container images"

    \b
    $ polyaxon registry push -f polyaxonfile.yaml --name=owner/name:v1 --description="Model description"
    """
    if not name:
        Printer.print_error(
            "Please provide a name to create a model version.",
            command_help="registry push",
            sys_exit=True,
        )
    owner, registry_name, version, is_version = get_info(None, name)
    tags = validate_tags(tags)

    if not owner or not registry_name or not version:
        Printer.print_error(
            "Please provide a valid model version with --name=owner/registry-name:version. "
        )
        sys.exit(1)

    polyaxon_client = PolyaxonClient()
    try:
        polyaxon_client.model_registry_v1.get_model_version(
            owner, registry_name, version
        )
        to_update = True
    except (ApiException, HTTPError):
        to_update = False

    if to_update:
        if not click.confirm(
            "A model version {}/{}:{} already exists. "
            "Do you want to push force this version?".format(
                owner, registry_name, version
            )
        ):
            click.echo("Existing without pushing model version.")
            sys.exit(1)

    try:
        registry_config = V1ModelVersion(
            name=version,
            description=description,
            tags=tags,
            run=run_uid,
        )
        if to_update:
            _version = polyaxon_client.model_registry_v1.update_model_version(
                owner,
                registry_name,
                version,
                registry_config,
            )
        else:
            _version = polyaxon_client.model_registry_v1.create_model_version(
                owner,
                registry_name,
                registry_config,
            )
    except (ApiException, HTTPError) as e:
        handle_cli_error(
            e, message="Could not create model version `{}`.".format(registry_name)
        )
        sys.exit(1)

    Printer.print_success(
        "Model version `{}` was created successfully.".format(_version.name)
    )
    click.echo(
        "You can view this model version on Polyaxon UI: {}".format(
            get_dashboard_url(
                subpath="{}/registry/{}/versions/{}".format(
                    owner, registry_name, _version.name
                )
            )
        )
    )


@registry.command()
@click.option(*OPTIONS_OWNER["args"], **OPTIONS_OWNER["kwargs"])
@click.option(*OPTIONS_MODEL_REGISTRY["args"], **OPTIONS_MODEL_REGISTRY["kwargs"])
@click.option(
    "--query",
    "-q",
    type=str,
    help="To filter the model registry/versions based on this query spec.",
)
@click.option(
    "--sort",
    "-s",
    type=str,
    help="To order the model registry/versions based on the sort spec.",
)
@click.option("--limit", type=int, help="To limit the list of model registry/versions.")
@click.option(
    "--offset", type=int, help="To offset the list of model registry/versions."
)
@clean_outputs
def ls(owner, model, query, sort, limit, offset):
    """List model registry/versions by owner or owner/model."""
    if owner and model:
        Printer.print_error(
            "Only an owner or a model is required, not both.", sys_exit=True
        )
    if model:
        owner, model_registry, model_version, is_version = get_info(model, None)
    elif not owner:
        owner = get_local_owner(is_cli=True)

    if not owner:
        Printer.print_error(
            "Please provide a valid owner --owner/-o or a model --model/-m."
        )
        sys.exit(1)

    def list_versions():
        model_info = "<owner: {}> <model: {}>".format(owner, model_registry)
        try:
            polyaxon_client = PolyaxonClient()
            params = get_query_params(
                limit=limit, offset=offset, query=query, sort=sort
            )
            response = polyaxon_client.model_registry_v1.list_model_versions(
                owner, model_registry, **params
            )
        except (ApiException, HTTPError) as e:
            message = "Could not get list of model version."
            handle_cli_error(e, message=message)
            sys.exit(1)

        meta = get_meta_response(response)
        if meta:
            Printer.print_header("Versions for {}".format(model_info))
            Printer.print_header("Navigation:")
            dict_tabulate(meta)
        else:
            Printer.print_header("No version found for {}".format(model_info))

        objects = list_dicts_to_tabulate(
            [o.to_dict() for o in response.results],
            humanize_values=True,
            exclude_attrs=[
                "uuid",
                "readme",
                "description",
                "owner",
                "owner",
                "role",
                "settings",
                "content",
                "live_state",
            ],
        )
        if objects:
            Printer.print_header("Model versions:")
            dict_tabulate(objects, is_list_dict=True)

    def list_models():
        try:
            polyaxon_client = PolyaxonClient()
            params = get_query_params(
                limit=limit, offset=offset, query=query, sort=sort
            )
            response = polyaxon_client.model_registry_v1.list_model_registries(
                owner, **params
            )
        except (ApiException, HTTPError) as e:
            message = "Could not get list of models."
            handle_cli_error(e, message=message)
            sys.exit(1)

        meta = get_meta_response(response)
        if meta:
            Printer.print_header("Models for owner {}".format(owner))
            Printer.print_header("Navigation:")
            dict_tabulate(meta)
        else:
            Printer.print_header("No model registry found for owner {}".format(owner))

        objects = list_dicts_to_tabulate(
            [o.to_dict() for o in response.results],
            humanize_values=True,
            exclude_attrs=[
                "uuid",
                "readme",
                "description",
                "owner",
                "role",
                "settings",
                "live_state",
            ],
        )
        if objects:
            Printer.print_header("Models:")
            dict_tabulate(objects, is_list_dict=True)

    if model:
        list_versions()
    else:
        list_models()


@registry.command()
@click.option(*OPTIONS_MODEL_REGISTRY["args"], **OPTIONS_MODEL_REGISTRY["kwargs"])
@click.option(*OPTIONS_MODEL_VERSION["args"], **OPTIONS_MODEL_VERSION["kwargs"])
@clean_outputs
def get(model, version):
    """Get info for a model registry by name, or owner/registry_name,
    or model version by name, name:tag, owner/name:tag.

    Examples:

    To get a default model registry:

    \b
    $ polyaxon registry get -h tensorboard

    To get by specific owner/name

    \b
    $ polyaxon registry get -p owner/my-model
    """
    owner, model_registry, model_version, is_version = get_info(model, version)

    try:
        polyaxon_client = PolyaxonClient()
        if is_version:
            response = polyaxon_client.model_registry_v1.get_model_version(
                owner, model_registry, model_version
            )
            get_model_version_details(response)
        else:
            response = polyaxon_client.model_registry_v1.get_model_registry(
                owner, model_registry
            )
            response.owner = owner
            get_entity_details(response, "Model registry")
    except (ApiException, HTTPError) as e:
        handle_cli_error(
            e,
            message="Could not get `{}`.".format(
                model_version if is_version else model_registry
            ),
            sys_exit=True,
        )


@registry.command()
@click.option(*OPTIONS_MODEL_REGISTRY["args"], **OPTIONS_MODEL_REGISTRY["kwargs"])
@click.option(*OPTIONS_MODEL_VERSION["args"], **OPTIONS_MODEL_VERSION["kwargs"])
@clean_outputs
def delete(model, version):
    """Delete a model registry or a model version."""
    owner, model_registry, model_version, is_version = get_info(model, version)
    full_entity = (
        "{}/{}:{}".format(owner, model_registry, model_version)
        if is_version
        else "{}/{}".format(owner, model_registry)
    )

    if not click.confirm(
        "Are sure you want to delete model {} `{}`".format(
            "version" if is_version else "registry", full_entity
        )
    ):
        click.echo("Existing without deleting model registry.")
        sys.exit(1)

    try:
        polyaxon_client = PolyaxonClient()
        if is_version:
            polyaxon_client.model_registry_v1.delete_model_version(
                owner, model_registry, model_version
            )
        else:
            polyaxon_client.model_registry_v1.delete_model_registry(
                owner, model_registry
            )
        Printer.print_success(
            "Model {} `{}` was delete successfully".format(
                "version" if is_version else "registry", full_entity
            )
        )
    except (ApiException, HTTPError) as e:
        handle_cli_error(
            e,
            message="Could not delete model {} `{}`.".format(
                "version" if is_version else "registry", full_entity
            ),
        )
        sys.exit(1)


@registry.command()
@click.option(*OPTIONS_MODEL_REGISTRY["args"], **OPTIONS_MODEL_REGISTRY["kwargs"])
@click.option(*OPTIONS_MODEL_VERSION["args"], **OPTIONS_MODEL_VERSION["kwargs"])
@click.option(
    "--name",
    type=str,
    help="Name of the model registry, must be unique for the same user.",
)
@click.option("--description", type=str, help="Description of the model registry.")
@click.option(
    "--tags", type=str, help="Tags of the run, comma separated values (optional)."
)
@click.option(
    "--private",
    type=bool,
    help="Set the visibility of the model registry to private/public.",
)
@clean_outputs
def update(model, version, name, description, tags, private):
    """Update model registry.

    Uses /docs/core/cli/#caching

    Example:

    \b
    $ polyaxon registry update foobar --description="Image Classification with DL using TensorFlow"

    \b
    $ polyaxon registry update mike1/foobar --description="Image Classification with DL using TensorFlow"

    \b
    $ polyaxon registry update --tags="foo, bar"
    """
    owner, model_registry, model_version, is_version = get_info(model, version)
    full_entity = (
        "{}/{}:{}".format(owner, model_registry, model_version)
        if is_version
        else "{}/{}".format(owner, model_registry)
    )

    update_dict = {}
    if name:
        update_dict["name"] = name

    if description:
        update_dict["description"] = description

    tags = validate_tags(tags)
    if tags:
        update_dict["tags"] = tags

    if private is not None:
        update_dict["is_public"] = not private

    if not update_dict:
        Printer.print_warning(
            "No argument was provided to update the model {}.".format(
                "version" if is_version else "registry"
            )
        )
        sys.exit(1)

    try:
        polyaxon_client = PolyaxonClient()
        if is_version:
            response = polyaxon_client.model_registry_v1.patch_model_version(
                owner, model_registry, model_version, body=update_dict
            )
            Printer.print_success("Model version updated.")
            get_model_version_details(response)
        else:
            response = polyaxon_client.model_registry_v1.patch_model_registry(
                owner, model_registry, body=update_dict
            )
            Printer.print_success("Model updated.")
            get_entity_details(response, "Model registry")
    except (ApiException, HTTPError) as e:
        handle_cli_error(
            e,
            message="Could not update model {} `{}`.".format(
                "version" if is_version else "registry", full_entity
            ),
        )
        sys.exit(1)


@registry.command()
@click.option(*OPTIONS_MODEL_REGISTRY["args"], **OPTIONS_MODEL_REGISTRY["kwargs"])
@click.option(*OPTIONS_MODEL_VERSION["args"], **OPTIONS_MODEL_VERSION["kwargs"])
@click.option(
    "--yes",
    "-y",
    is_flag=True,
    default=False,
    help="Automatic yes to prompts. "
    'Assume "yes" as answer to all prompts and run non-interactively.',
)
@click.option(
    "--url",
    is_flag=True,
    default=False,
    help="Print the url of the dashboard for this model registry.",
)
@clean_outputs
def dashboard(model, version, yes, url):
    """Open this operation's dashboard details in browser."""
    owner, model_registry, model_version, is_version = get_info(model, version)
    subpath = (
        "{}/registry/{}/versions?version={}".format(
            owner, model_registry, model_version
        )
        if is_version
        else "{}/registry/{}".format(owner, model_registry)
    )

    registry_url = get_dashboard_url(subpath=subpath)
    if url:
        Printer.print_header("The dashboard is available at: {}".format(registry_url))
        sys.exit(0)
    if not yes:
        click.confirm(
            "Dashboard page will now open in your browser. Continue?",
            abort=True,
            default=True,
        )
    click.launch(registry_url)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest.mock import MagicMock

from polyaxon.utils.urls_utils import (
    get_fqn_run_url,
    get_owner_project_url,
    get_owner_url,
    get_project_url,
    get_run_health_url,
    get_run_url,
)
from tests.utils import BaseTestCase


class TestUrlsUtils(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.user = MagicMock(username="foo")
        unique_name = f"{self.user.username}.project-test"
        self.project = MagicMock(
            id=1, name="project-test", user=self.user, unique_name=unique_name
        )
        unique_name = f"{unique_name}.uuid"
        self.run = MagicMock(
            uuid="uuid", id=2, project=self.project, unique_name=unique_name
        )

    def test_get_owner_url(self):
        self.assertEqual(get_owner_url("foo"), "/foo")

    def test_get_project_url(self):
        unique_name = self.project.unique_name
        self.assertEqual(
            get_project_url(unique_name=unique_name),
            "/{}".format(unique_name.replace(".", "/")),
        )

    def test_get_owner_project_url(self):
        self.assertEqual(
            get_owner_project_url(owner="foo", project_name="bar"), "/foo/bar"
        )

    def test_get_run_url(self):
        self.assertEqual(
            get_run_url(
                owner="foo", project_name="project-test", run_uuid=self.run.uuid
            ),
            "/foo/project-test/runs/uuid",
        )

    def test_get_fqn_run_url(self):
        unique_name = self.run.unique_name
        self.assertEqual(
            get_fqn_run_url(unique_name=unique_name), "/foo/project-test/runs/uuid"
        )

    def test_get_run_health_url(self):
        self.assertEqual(
            get_run_health_url(unique_name=self.run.unique_name),
            "{}/_heartbeat".format(get_fqn_run_url(self.run.unique_name)),
        )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from marshmallow import ValidationError

from polyaxon.polyflow import V1EventKind, V1Hook, V1Param, V1RunKind
from polyaxon.polyflow.operations import V1Operation
from tests.utils import BaseTestCase


@pytest.mark.ops_mark
class TestV1Operations(BaseTestCase):
    def test_op_raises_for_template_action_event(self):
        config_dict = {"hubRef": "foo", "urlRef": "bar"}
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {"pathRef": "bar", "urlRef": "foo"}
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {"hubRef": "bar", "urlRef": "foo"}
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

    def test_op_passes_for_template_hub(self):
        config_dict = {"hubRef": "bar"}
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        config_dict = {"pathRef": "bar"}
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        config_dict = {"urlRef": "bar"}
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        config_dict = {"dagRef": "bar"}
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_validation(self):
        config_dict = {"hubRef": "foo", "concurrency": "foo"}
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {"hubRef": "foo", "dependencies": "foo"}
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {
            "hubRef": "foo",
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        config_dict = {"hubRef": "foo"}
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        config_dict = {
            "hubRef": "foo",
            "dependencies": [{"name": "foo"}, {"name": "bar"}],  # Wrong
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

    def test_op_with_embedded_job_template(self):
        # Embedding a bad template
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {
                "not_supported_key": "build-template",
                "tags": ["kaniko"],
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}},
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        # Embedding a template without container
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {"name": "build-template", "tags": ["kaniko"]},
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        # Embedding a correct template
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {
                "name": "build-template",
                "tags": ["kaniko"],
                "run": {
                    "kind": V1RunKind.JOB,
                    "container": {"image": "test"},
                    "init": [{"connection": "foo", "container": {"image": "dev"}}],
                },
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_op_with_embedded_service_template(self):
        # Embedding a bad template
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "runPatch": {
                "kind": V1RunKind.JOB,
                "init": [{"connection": "foo", "git": {"revision": "dev"}}],
            },
            "component": {
                "not_supported_key": "build-template",
                "tags": ["lab"],
                "run": {
                    "kind": V1RunKind.JOB,
                    "container": {"image": "jupyterlab"},
                    "environment": {
                        "nodeSelector": {"polyaxon.com": "node_for_notebook_jobs"},
                        "config_maps": [{"name": "config_map1"}],
                    },
                },
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        # Embedding a correct template
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "runPatch": {
                "kind": V1RunKind.JOB,
                "init": [
                    {"connection": "git-repo1", "git": {"revision": "dev"}},
                    {"connection": "git-repo1", "container": {"args": "--branch=dev"}},
                ],
            },
            "component": {
                "name": "build-template",
                "tags": ["lab"],
                "run": {
                    "kind": V1RunKind.JOB,
                    "environment": {
                        "nodeSelector": {"polyaxon.com": "node_for_notebook_jobs"}
                    },
                    "container": {"image": "jupyterlab"},
                },
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_op_with_embedded_pipeline_template(self):
        # Embedding a bad template
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {
                "version": 1.1,
                "key_not_supported": [
                    {"dagRef": "build-template", "name": "A"},
                    {"dagRef": "job-template", "name": "B", "dependencies": ["A"]},
                ],
                "run": {
                    "kind": "dag",
                    "operations": [
                        {
                            "kind": "operation",
                            "name": "job-template",
                            "component": {
                                "run": {
                                    "kind": V1RunKind.JOB,
                                    "container": {"image": "test"},
                                }
                            },
                        },
                        {
                            "kind": "operation",
                            "name": "build-template",
                            "tags": ["kaniko"],
                            "init": [{"connection": "foo", "git": {"branch": "dev"}}],
                        },
                    ],
                },
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        # Embedding a template without container
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {
                "run": {
                    "kind": "dag",
                    "operations": [
                        {
                            "kind": "operation",
                            "name": "A",
                            "component": {
                                "run": {
                                    "kind": V1RunKind.JOB,
                                    "container": {"image": "test"},
                                }
                            },
                        },
                        {
                            "kind": "operation",
                            "name": "B",
                            "dependencies": ["A"],
                            "tags": ["kaniko"],
                        },
                    ],
                }
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        # Embedding a correct template
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {
                "run": {
                    "kind": "dag",
                    "operations": [
                        {
                            "kind": "operation",
                            "name": "A",
                            "component": {
                                "run": {
                                    "kind": V1RunKind.JOB,
                                    "container": {"image": "test"},
                                }
                            },
                        },
                        {
                            "kind": "operation",
                            "name": "B",
                            "dependencies": ["A"],
                            "tags": ["kaniko"],
                            "component": {
                                "run": {
                                    "kind": V1RunKind.JOB,
                                    "init": [
                                        {
                                            "connection": "foo",
                                            "container": {"image": "foo"},
                                        }
                                    ],
                                    "container": {"image": "test"},
                                }
                            },
                        },
                    ],
                }
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_op_with_embedded_non_supported_kind_template(self):
        config_dict = {
            "dependencies": ["foo", "bar"],
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "trigger": "all_succeeded",
            "component": {
                "run": {"run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}}
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

    def test_op_and_hooks(self):
        config_dict = {
            "hooks": [
                {"trigger": "succeeded", "connection": "connection1", "hubRef": "ref1"},
                {"connection": "connection1", "hubRef": "ref2"},
            ],
            "component": {
                "run": {
                    "kind": V1RunKind.JOB,
                    "environment": {
                        "nodeSelector": {"polyaxon.com": "node_for_notebook_jobs"}
                    },
                    "container": {"image": "jupyterlab"},
                },
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_op_and_joins(self):
        config_dict = {
            "joins": [
                {
                    "sort": 2,
                    "params": {
                        "u": {"value": "outputs.value"},
                    },
                },
            ],
            "component": {
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {
            "joins": [
                {
                    "query": "dummy: query1",
                    "limit": "foo",
                },
            ],
            "component": {
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {
            "joins": [
                {
                    "query": "dummy: query1",
                    "sort": "dummy sort1",
                    "params": {
                        "u": {"value": "outputs.value"},
                    },
                },
                {
                    "query": "dummy: query1",
                    "sort": "-dummy-sort2",
                    "params": {
                        "u": {"value": "artifacts"},
                    },
                },
            ],
            "component": {
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_op_and_events(self):
        config_dict = {
            "events": [
                {"kinds": ["some-value"], "ref": "test"},
            ],
            "component": {
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {
            "events": [
                {"kinds": [V1EventKind.RUN_STATUS_SCHEDULED]},
            ],
            "component": {
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}
            },
        }
        with self.assertRaises(ValidationError):
            V1Operation.from_dict(config_dict)

        config_dict = {
            "events": [
                {"kinds": [V1EventKind.RUN_STATUS_SCHEDULED], "ref": "test.r1"},
            ],
            "component": {
                "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}}
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_op_template(self):
        config_dict = {
            "hooks": [
                {"trigger": "succeeded", "connection": "connection1", "hubRef": "ref1"},
                {"connection": "connection1", "hubRef": "ref2"},
            ],
            "events": [
                {"kinds": [V1EventKind.RUN_STATUS_SCHEDULED], "ref": "test.r1"},
                {"kinds": [V1EventKind.RUN_NEW_ARTIFACTS], "ref": "test.r2"},
            ],
            "joins": [
                {
                    "query": "dummy: query1",
                    "sort": "dummy sort1",
                    "params": {
                        "u": {"value": "outputs.value"},
                    },
                },
                {
                    "query": "dummy: query1",
                    "sort": "-dummy-sort2",
                    "params": {
                        "u": {"value": "artifacts"},
                    },
                },
            ],
            "component": {
                "run": {
                    "kind": V1RunKind.JOB,
                    "environment": {
                        "nodeSelector": {"polyaxon.com": "node_for_notebook_jobs"}
                    },
                    "container": {"image": "jupyterlab"},
                },
            },
            "template": {
                "description": "This is a template, check the fields",
                "fields": ["actions[1].hubRef", "hooks[0].trigger"],
            },
        }
        config = V1Operation.from_dict(config_dict)
        assert config.to_dict() == config_dict

    def test_from_hook(self):
        config_dict = {
            "connection": "test-connection",
            "trigger": "succeeded",
            "hubRef": "foo",
            "conditions": "foo > bar",
            "params": {"param1": {"value": "foo"}, "param2": {"value": "bar"}},
            "presets": ["pre1", "pre2"],
        }
        hook = V1Hook.from_dict(config_dict)

        op = V1Operation.from_hook(hook, None, None, None)
        assert op.run_patch["connections"] == [hook.connection]
        assert op.hub_ref == hook.hub_ref
        assert op.params == hook.params
        assert op.presets == hook.presets

        op = V1Operation.from_hook(hook, {"in1": "v1"}, {"out1": "v1"}, {"c1": "v1"})
        assert op.run_patch["connections"] == [hook.connection]
        assert op.hub_ref == hook.hub_ref
        assert op.params == {
            **hook.params,
            "inputs": V1Param(value={"in1": "v1"}, context_only=True),
            "outputs": V1Param(value={"out1": "v1"}, context_only=True),
            "condition": V1Param(value={"c1": "v1"}, context_only=True),
        }
        assert op.presets == hook.presets



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.auxiliaries import V1PolyaxonInitContainer, get_init_resources
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import V1GitConnection
from polyaxon.containers.contexts import CONTEXT_MOUNT_ARTIFACTS
from polyaxon.containers.names import INIT_GIT_CONTAINER_PREFIX, generate_container_name
from polyaxon.exceptions import PolypodException
from polyaxon.polyflow import V1Plugins
from polyaxon.polypod.common import constants
from polyaxon.polypod.common.env_vars import get_connection_env_var
from polyaxon.polypod.common.mounts import (
    get_auth_context_mount,
    get_connections_context_mount,
)
from polyaxon.polypod.common.volumes import get_volume_name
from polyaxon.polypod.init.git import get_git_init_container, get_repo_context_args
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.schemas.types import V1ConnectionType
from tests.utils import BaseTestCase


@pytest.mark.polypod_mark
class TestInitGit(BaseTestCase):
    def test_get_repo_context_args_requires_from_image(self):
        with self.assertRaises(PolypodException):
            get_repo_context_args(name=None, url=None, revision=None, mount_path=None)

    def test_get_repo_context_args_with_none_values(self):
        args = get_repo_context_args(
            name="user/repo1",
            url="http://foo.com",
            revision=None,
            mount_path="/somepath",
        )
        assert args == ["--repo-path=/somepath/user/repo1", "--url=http://foo.com"]

    def test_get_repo_context_args(self):
        args = get_repo_context_args(
            name="user/repo1",
            url="http://foo.com",
            revision="00b9d2ea01c40f58d6b4051319f9375675a43c02",
            mount_path="/somepath",
        )
        assert args == [
            "--repo-path=/somepath/user/repo1",
            "--url=http://foo.com",
            "--revision=00b9d2ea01c40f58d6b4051319f9375675a43c02",
        ]

        args = get_repo_context_args(
            name="user/repo1",
            url="http://foo.com",
            revision="dev",
            mount_path="/somepath",
        )
        assert args == [
            "--repo-path=/somepath/user/repo1",
            "--url=http://foo.com",
            "--revision=dev",
        ]

        args = get_repo_context_args(
            name="user/repo1",
            url="http://foo.com",
            revision="00b9d2ea01c40f58d6b4051319f9375675a43c02",
            mount_path="/somepath",
        )
        assert args == [
            "--repo-path=/somepath/user/repo1",
            "--url=http://foo.com",
            "--revision=00b9d2ea01c40f58d6b4051319f9375675a43c02",
        ]

    def test_get_git_init_container_raises_for_missing_info(self):
        with self.assertRaises(PolypodException):
            get_git_init_container(
                polyaxon_init=V1PolyaxonInitContainer(), connection=None, contexts=None
            )

        with self.assertRaises(PolypodException):
            get_git_init_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/test"),
                connection=None,
                mount_path=None,
                contexts=None,
            )

    def test_get_git_init_container(self):
        connection = V1ConnectionType(
            name="user/foo",
            kind=V1ConnectionKind.GIT,
            schema=V1GitConnection(url="foo.com"),
        )
        container = get_git_init_container(
            polyaxon_init=V1PolyaxonInitContainer(image="foo", image_tag=""),
            connection=connection,
            contexts=PluginsContextsSpec.from_config(V1Plugins(auth=True)),
        )
        assert (
            generate_container_name(INIT_GIT_CONTAINER_PREFIX, connection.name, False)
            in container.name
        )
        assert container.image == "foo"
        assert container.image_pull_policy is None
        assert container.command == ["polyaxon", "initializer", "git"]
        assert container.env == get_connection_env_var(
            connection=connection, secret=None
        )
        assert container.resources == get_init_resources()
        assert container.volume_mounts == [
            get_connections_context_mount(
                name=constants.CONTEXT_VOLUME_ARTIFACTS,
                mount_path=CONTEXT_MOUNT_ARTIFACTS,
            ),
            get_auth_context_mount(read_only=True),
        ]

        container = get_git_init_container(
            polyaxon_init=V1PolyaxonInitContainer(
                image="init/init", image_tag="", image_pull_policy="IfNotPresent"
            ),
            connection=connection,
            contexts=PluginsContextsSpec.from_config(V1Plugins(auth=True)),
        )
        assert (
            generate_container_name(INIT_GIT_CONTAINER_PREFIX, connection.name, False)
            in container.name
        )
        assert container.image == "init/init"
        assert container.image_pull_policy == "IfNotPresent"
        assert container.command == ["polyaxon", "initializer", "git"]
        assert container.args == [
            "--repo-path={}/{}".format(CONTEXT_MOUNT_ARTIFACTS, connection.name),
            "--url={}".format(connection.schema.url),
        ]
        assert container.resources == get_init_resources()
        assert container.volume_mounts == [
            get_connections_context_mount(
                name=constants.CONTEXT_VOLUME_ARTIFACTS,
                mount_path=CONTEXT_MOUNT_ARTIFACTS,
            ),
            get_auth_context_mount(read_only=True),
        ]

        connection = V1ConnectionType(
            name="user/foo",
            kind=V1ConnectionKind.GIT,
            schema=V1GitConnection(
                url="foo.com", revision="00b9d2ea01c40f58d6b4051319f9375675a43c02"
            ),
        )
        container = get_git_init_container(
            polyaxon_init=V1PolyaxonInitContainer(
                image="init/init", image_tag="", image_pull_policy="IfNotPresent"
            ),
            connection=connection,
            mount_path="/somepath",
            contexts=PluginsContextsSpec.from_config(V1Plugins(auth=True)),
        )
        assert (
            generate_container_name(INIT_GIT_CONTAINER_PREFIX, connection.name, False)
            in container.name
        )
        assert container.image == "init/init"
        assert container.image_pull_policy == "IfNotPresent"
        assert container.command == ["polyaxon", "initializer", "git"]
        assert container.args == [
            "--repo-path=/somepath/{}".format(connection.name),
            "--url={}".format(connection.schema.url),
            "--revision=00b9d2ea01c40f58d6b4051319f9375675a43c02",
        ]
        assert container.resources == get_init_resources()
        assert container.volume_mounts == [
            get_connections_context_mount(
                name=get_volume_name("/somepath"), mount_path="/somepath"
            ),
            get_auth_context_mount(read_only=True),
        ]

        connection = V1ConnectionType(
            name="user/foo",
            kind=V1ConnectionKind.GIT,
            schema=V1GitConnection(
                url="foo.com",
                revision="00b9d2ea01c40f58d6b4051319f9375675a43c02",
                flags=["--falg1", "--flag2=test", "k=v"],
            ),
        )
        container = get_git_init_container(
            polyaxon_init=V1PolyaxonInitContainer(
                image="init/init", image_tag="", image_pull_policy="IfNotPresent"
            ),
            connection=connection,
            mount_path="/somepath",
            contexts=PluginsContextsSpec.from_config(V1Plugins(auth=True)),
        )
        assert (
            generate_container_name(INIT_GIT_CONTAINER_PREFIX, connection.name, False)
            in container.name
        )
        assert container.image == "init/init"
        assert container.image_pull_policy == "IfNotPresent"
        assert container.command == ["polyaxon", "initializer", "git"]
        assert container.args == [
            "--repo-path=/somepath/{}".format(connection.name),
            "--url={}".format(connection.schema.url),
            "--revision=00b9d2ea01c40f58d6b4051319f9375675a43c02",
            '--flags=["--falg1","--flag2=test","k=v"]',
        ]
        assert container.resources == get_init_resources()
        assert container.volume_mounts == [
            get_connections_context_mount(
                name=get_volume_name("/somepath"), mount_path="/somepath"
            ),
            get_auth_context_mount(read_only=True),
        ]



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint:disable=protected-access
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.exceptions import PolyaxonNotificationException
from polyaxon.notifiers.pagerduty_webhook import PagerDutyWebHookNotifier
from tests.test_notifiers.test_webhook_notification import TestWebHookNotification


class TestPagerDutyWebHook(TestWebHookNotification):
    webhook = PagerDutyWebHookNotifier

    def test_attrs(self):
        assert self.webhook.notification_key == V1ConnectionKind.PAGERDUTY
        assert self.webhook.name == "PagerDuty WebHook"

    def test_validate_config(self):
        assert self.webhook._validate_config(
            {
                "url": "http://pagerduty.com/webhook/foo",
                "method": "post",
                "service_key": "foo",
            }
        ) == [
            {
                "url": "http://pagerduty.com/webhook/foo",
                "method": "POST",
                "service_key": "foo",
            }
        ]

        assert self.webhook._validate_config(
            [
                {
                    "url": "http://pagerduty.com/webhook/foo",
                    "method": "post",
                    "service_key": "foo",
                },
                {"url": "http://pagerduty.com/webhook/bar", "method": "GET"},
            ]
        ) == [
            {
                "url": "http://pagerduty.com/webhook/foo",
                "method": "POST",
                "service_key": "foo",
            },
            {"url": "http://pagerduty.com/webhook/bar", "method": "GET"},
        ]

    def test_get_config(self):
        assert self.webhook.get_config(
            {"url": "http://foo.com/webhook", "method": "post", "service_key": "foo"}
        ) == [{"url": "http://foo.com/webhook", "method": "POST", "service_key": "foo"}]
        assert self.webhook.get_config(
            [
                {
                    "url": "http://foo.com/webhook",
                    "method": "post",
                    "service_key": "foo",
                },
                {
                    "url": "http://bar.com/webhook",
                    "method": "GET",
                    "service_key": "bar",
                },
            ]
        ) == [
            {"url": "http://foo.com/webhook", "method": "POST", "service_key": "foo"},
            {"url": "http://bar.com/webhook", "method": "GET", "service_key": "bar"},
        ]

    def test_prepare(self):
        with self.assertRaises(PolyaxonNotificationException):
            self.webhook._prepare(None)
        with self.assertRaises(PolyaxonNotificationException):
            self.webhook._prepare({})

        context = {"title": "title", "text": "text"}
        assert self.webhook._prepare(context) == {
            "event_type": context.get("event_type"),
            "description": context.get("description"),
            "details": context.get("details"),
            "incident_key": context.get("incident_key"),
            "client": "Polyaxon",
            "client_url": "https://polyaxon.com",
            "contexts": context.get("contexts"),
        }


del TestWebHookNotification



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def get_project_instance(owner: str, project: str) -> str:
    return "{}.{}".format(owner, project)


def get_run_instance(owner: str, project: str, run_uuid: str) -> str:
    return "{}.{}.runs.{}".format(owner, project, run_uuid)


def get_notifier_instance(owner: str, project: str, run_uuid: str) -> str:
    return "{}.{}.notifiers.{}".format(owner, project, run_uuid)


def get_watchdog_instance(owner: str, project: str, run_uuid: str) -> str:
    return "{}.{}.watchdog.{}".format(owner, project, run_uuid)


def get_tuner_instance(owner: str, project: str, run_uuid: str) -> str:
    return "{}.{}.tuners.{}".format(owner, project, run_uuid)


def get_cleaner_instance(owner: str, project: str, run_uuid: str) -> str:
    return "{}.{}.cleaners.{}".format(owner, project, run_uuid)


def get_resource_name(run_uuid: str) -> str:
    return "plx-operation-{}".format(run_uuid)


def get_notifier_resource_name(run_uuid: str) -> str:
    return "plx-notifier-{}".format(run_uuid)


def get_watchdog_resource_name(run_uuid: str) -> str:
    return "plx-watchdog-{}".format(run_uuid)


def get_tuner_resource_name(run_uuid: str) -> str:
    return "plx-tuner-{}".format(run_uuid)


def get_cleaner_resource_name(run_uuid: str) -> str:
    return "plx-cleaner-{}".format(run_uuid)


def get_resource_name_for_kind(run_uuid: str, run_kind: str = None) -> str:
    if run_kind == "cleaner":
        return get_cleaner_resource_name(run_uuid)
    if run_kind == "tuner":
        return get_tuner_resource_name(run_uuid)
    if run_kind == "watchdog":
        return get_watchdog_resource_name(run_uuid)
    if run_kind == "notifier":
        return get_notifier_resource_name(run_uuid)
    # Operation
    return get_resource_name(run_uuid)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow.exceptions import ValidationError

from polyaxon.connections.schemas import V1K8sResourceSchema, validate_k8s_resource
from tests.utils import BaseTestCase


class TestSecretResourceValidation(BaseTestCase):
    def test_claim_connect_config(self):
        config_dict = {}
        with self.assertRaises(ValidationError):
            V1K8sResourceSchema.from_dict(config_dict)

        with self.assertRaises(ValidationError):
            validate_k8s_resource(config_dict)

        config_dict = {"name": "sdf"}
        V1K8sResourceSchema.from_dict(config_dict)
        validate_k8s_resource(config_dict)

        config_dict = {"name": "sdf", "items": ["foo"], "mountPath": "/bar"}
        config = V1K8sResourceSchema.from_dict(config_dict)
        assert config.to_dict() == config_dict
        validate_k8s_resource(config_dict)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import getpass
import os
import platform
import socket
import sys

from polyaxon.logger import logger


def is_notebook():
    return "ipykernel" in sys.modules


def get_filename():
    if is_notebook():
        return "notebook"
    try:
        return os.path.basename(__file__)
    except Exception as e:
        logger.debug("Could not detect filename, %s", e)
        return "not found"


def get_module_path():
    try:
        return os.path.dirname(os.path.realpath("__file__"))
    except Exception as e:
        logger.debug("Could not detect module path, %s", e)
        return "not found"


def get_user():
    try:
        return getpass.getuser()
    except Exception as e:
        logger.debug("Could not detect installed packages, %s", e)
        return "unknown"


def get_run_env():
    import pkg_resources

    def get_packages():
        try:
            installed_packages = [d for d in pkg_resources.working_set]  # noqa
            return sorted(
                ["{}=={}".format(pkg.key, pkg.version) for pkg in installed_packages]
            )
        except Exception as e:
            logger.debug("Could not detect installed packages, %s", e)
            return []

    try:
        version = pkg_resources.get_distribution("polyaxon").version
    except pkg_resources.DistributionNotFound:
        version = ""
    return {
        "pid": os.getpid(),
        "hostname": socket.gethostname(),
        "os": platform.platform(aliased=True),
        "system": platform.system(),
        "python_version_verbose": sys.version,
        "python_version": platform.python_version(),
        "user": get_user(),
        "client_version": version,
        "sys.argv": sys.argv,
        "is_notebook": is_notebook(),
        "filename": get_filename(),
        "module_path": get_module_path(),
        "packages": get_packages(),
    }



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import tempfile

from polyaxon import settings, types
from polyaxon.auxiliaries import (
    get_default_init_container,
    get_default_sidecar_container,
)
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    V1BucketConnection,
    V1HostConnection,
    V1K8sResourceSchema,
)
from polyaxon.managers.agent import AgentConfigManager
from polyaxon.polyaxonfile.specs import kinds
from polyaxon.polyboard.artifacts import V1ArtifactKind
from polyaxon.polyflow import V1CompiledOperation, V1RunKind
from polyaxon.polypod.compiler.lineage.io_collector import collect_io_artifacts
from polyaxon.polypod.compiler.resolver import BaseResolver
from polyaxon.schemas.cli.agent_config import AgentConfig
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType
from tests.utils import BaseTestCase


@pytest.mark.polypod_mark
class TestLineageResolver(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.compiled_operation = V1CompiledOperation.read(
            {
                "version": 1.1,
                "kind": kinds.COMPILED_OPERATION,
                "inputs": [
                    {
                        "name": "param1",
                        "type": types.STR,
                        "value": "test",
                        "isOptional": "true",
                    },
                    {
                        "name": "param1",
                        "type": types.IMAGE,
                        "isOptional": "true",
                        "value": "repo1",
                        "connection": "connection1",
                    },
                    {
                        "name": "param1",
                        "type": types.IMAGE,
                        "isOptional": "true",
                        "value": "repo2",
                        "connection": "connection2",
                    },
                ],
                "outputs": [
                    {
                        "name": "repo2",
                        "type": types.IMAGE,
                        "isOptional": "true",
                        "value": "repo3",
                        "connection": "connection1",
                    }
                ],
                "run": {
                    "kind": V1RunKind.JOB,
                    "connections": {"test_s3", "connection1", "connection2"},
                    "container": {"image": "test"},
                },
            }
        )

    def test_collector_without_connections(self):
        artifacts = collect_io_artifacts(
            compiled_operation=self.compiled_operation, connection_by_names={}
        )
        assert len(artifacts) == 3
        assert {a.is_input for a in artifacts} == {True, False}
        assert {a.kind for a in artifacts} == {V1ArtifactKind.DOCKER_IMAGE}
        assert {a.connection for a in artifacts} == {"connection1", "connection2"}
        assert {a.summary.get("image") for a in artifacts} == {
            "repo1",
            "repo2",
            "repo3",
        }

    def test_collector_with_connections(self):
        connection1 = V1ConnectionType(
            name="connection1",
            kind=V1ConnectionKind.REGISTRY,
            schema=V1HostConnection(url="localhost:5000"),
            secret="test",
        )
        artifacts = collect_io_artifacts(
            compiled_operation=self.compiled_operation,
            connection_by_names={"connection1": connection1},
        )
        assert len(artifacts) == 3
        assert {a.is_input for a in artifacts} == {True, False}
        assert {a.kind for a in artifacts} == {V1ArtifactKind.DOCKER_IMAGE}
        assert {a.connection for a in artifacts} == {"connection1", "connection2"}
        assert {a.summary.get("image") for a in artifacts} == {
            "localhost:5000/repo1",
            "repo2",
            "localhost:5000/repo3",
        }

    def test_resolve_connections_with_invalid_config(self):
        fpath = tempfile.mkdtemp()
        AgentConfigManager.CONFIG_PATH = fpath
        secret1 = V1K8sResourceType(
            name="secret1",
            schema=V1K8sResourceSchema(name="secret1"),
            is_requested=True,
        )
        secret2 = V1K8sResourceType(
            name="secret2",
            schema=V1K8sResourceSchema(name="secret2"),
            is_requested=True,
        )
        artifacts_store = V1ConnectionType(
            name="test_s3",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3//:foo"),
            secret=secret1.schema,
        )
        connection1 = V1ConnectionType(
            name="connection1",
            kind=V1ConnectionKind.REGISTRY,
            schema=V1HostConnection(url="localhost:5000"),
            secret=secret2.schema,
        )
        connection2 = V1ConnectionType(
            name="connection2",
            kind=V1ConnectionKind.REGISTRY,
        )
        settings.AGENT_CONFIG = AgentConfig(
            namespace="foo",
            artifacts_store=artifacts_store,
            connections=[connection1, connection2],
        )

        resolver = BaseResolver(
            run=None,
            compiled_operation=self.compiled_operation,
            owner_name="user",
            project_name="p1",
            project_uuid=None,
            run_name="j1",
            run_uuid=None,
            run_path="test",
            params=None,
        )
        resolver.resolve_connections()
        assert resolver.namespace == "foo"
        assert resolver.connection_by_names == {
            artifacts_store.name: artifacts_store,
            connection1.name: connection1,
            connection2.name: connection2,
        }
        assert resolver.artifacts_store == artifacts_store
        assert resolver.polyaxon_sidecar == get_default_sidecar_container()
        assert resolver.polyaxon_init == get_default_init_container()
        resolver.resolve_artifacts_lineage()
        assert len(resolver.artifacts) == 3
